#!/usr/bin/env python3

from os import environ, path
from subprocess import call

prefix = environ.get('MESON_INSTALL_PREFIX', '/usr/local')
datadir = path.join(prefix, 'share')
destdir = environ.get('DESTDIR', '')

if not destdir:
    print('Updating icon cache...')
    call(['gtk-update-icon-cache', '-qtf', path.join(datadir, 'icons', 'hicolor')])

    print('Updating desktop database')
    call(['update-desktop-database', '-q', path.join(datadir, 'applications')])

    print('Compiling GSettings schemas...')
    call(['glib-compile-schemas', path.join(datadir, 'glib-2.0', 'schemas')])



# Copyright 2021 - 2022, Martijn Braam and the OpenAtem contributors
# SPDX-License-Identifier: GPL-3.0-only
import ctypes
import glob
import json
import logging
import os.path
import threading
import time
import traceback
import uuid
from datetime import datetime

import gi

from gtk_switcher.debugger import DebuggerWindow
from gtk_switcher.routing import Routing
from gtk_switcher.videohubconnection import VideoHubConnection
from pyatem.hexdump import hexdump

from gtk_switcher.audio import AudioPage
from gtk_switcher.camera import CameraPage
from gtk_switcher.decorators import field, call_fields
from gtk_switcher.macroeditor import MacroEditorWindow
from gtk_switcher.media import MediaPage
from gtk_switcher.connectionwindow import ConnectionWindow
from gtk_switcher.switcher import SwitcherPage
from pyatem.command import ProgramInputCommand, PreviewInputCommand, AutoCommand, TransitionPositionCommand, \
    InputPropertiesCommand
from pyatem.protocol import AtemProtocol
import pyatem.field as fieldmodule

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GLib, GObject, Gio, Gdk

gi.require_version('Handy', '1')
from gi.repository import Handy


class AtemConnection(threading.Thread):
    def __init__(self, callback, disconnected, transfer_progress, download_done, connected, upload_done,
                 upload_progress):
        threading.Thread.__init__(self)
        self.name = 'Connection'
        self.callback = callback
        self.disconnected = disconnected
        self._connected = connected
        self.transfer_progress = transfer_progress
        self.download_done = download_done
        self.upload_done = upload_done
        self.upload_progress = upload_progress
        self.atem = None
        self.ip = None
        self.stop = False
        self.connected = False
        self.log = logging.getLogger('AtemConnection')

    def run(self):
        # Don't run if the ip isn't set yet
        if self.ip is None or self.ip == '0.0.0.0':
            if AtemProtocol.usb_exists():
                self.log.info(f'Connect to USB device')
                try:
                    self.mixer = AtemProtocol(usb="auto")
                except PermissionError:
                    self.log.error("Could not connect to USB device: permission denied")
                    self.log.error("The udev rules for this ATEM device might be missing on your system")
                    self.do_permission_error()
                    return
            else:
                self.log.error(f'Invalid connection parameter')
                return
        else:
            self.log.info(f'Connect to {self.ip}')
            self.mixer = AtemProtocol(self.ip)
        self.mixer.on('change', self.do_callback)
        self.mixer.on('connected', self.do_connected)
        self.mixer.on('disconnected', self.do_disconnected)
        self.mixer.on('transfer-progress', self.do_transfer_progress)
        self.mixer.on('download-done', self.do_download_done)
        self.mixer.on('upload-done', self.do_upload_done)
        self.mixer.on('upload-progress', self.do_upload_progress)
        try:
            self.mixer.connect()
        except ConnectionError as e:
            self.log.error(f"Could not connect to {self.ip}: {e}")
            return
        while not self.stop:
            try:
                self.mixer.loop()
            except Exception as e:
                traceback.print_exc()
                self.log.error(repr(e))

    def do_callback(self, *args, **kwargs):
        GLib.idle_add(self.callback, *args, **kwargs)

    def do_disconnected(self):
        self.connected = False
        GLib.idle_add(self.disconnected, "disconnected")

    def do_permission_error(self):
        self.connected = False
        GLib.idle_add(self.disconnected, "permission")

    def do_connected(self):
        self.connected = True
        GLib.idle_add(self._connected)

    def do_transfer_progress(self, store, slot, progress):
        GLib.idle_add(self.transfer_progress, store, slot, progress)

    def do_download_done(self, store, slot, data):
        GLib.idle_add(self.download_done, store, slot, data)

    def do_upload_progress(self, store, slot, percent, done, size):
        GLib.idle_add(self.upload_progress, store, slot, percent, done, size)

    def do_upload_done(self, store, slot):
        GLib.idle_add(self.upload_done, store, slot)

    def get_id(self):

        # returns id of the respective thread
        if hasattr(self, '_thread_id'):
            return self._thread_id
        for id, thread in threading._active.items():
            if thread is self:
                return id

    def die(self):
        thread_id = self.get_id()
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(thread_id,
                                                         ctypes.py_object(SystemExit))
        if res > 1:
            ctypes.pythonapi.PyThreadState_SetAsyncExc(thread_id, 0)
            self.log.error('Exception raise failure')


class AtemWindow(SwitcherPage, MediaPage, AudioPage, CameraPage):
    def __init__(self, application, args):
        self.application = application
        self.args = args

        self.debug = args.debug
        self.log_aw = logging.getLogger('AtemWindow')

        Handy.init()

        self.settings = Gio.Settings.new('nl.brixit.Switcher')
        self.settings.connect('changed::switcher-ip', self.on_switcher_ip_changed)
        self.settings.connect('changed::connections', self.on_connection_settings_changed)

        builder = Gtk.Builder()
        builder.set_translation_domain("openswitcher")
        builder.add_from_resource('/nl/brixit/switcher/ui/mixer.glade')
        builder.connect_signals(self)
        css = Gio.resources_lookup_data("/nl/brixit/switcher/ui/style.css", 0)

        self.provider = Gtk.CssProvider()
        self.provider.load_from_data(css.get_data())

        self.window = builder.get_object("main_window")
        self.window.set_application(self.application)
        self.headerbar = builder.get_object("headerbar")
        self.header_context_stack = builder.get_object("header_context_stack")
        self.macro_status_index = builder.get_object("macro_status_index")

        # Load requested view
        self.mainstack = builder.get_object("mainstack")
        self.connectionstack = builder.get_object("connectionstack")
        self.mainstack.set_visible_child_name(args.view)
        self.connectionstack.set_visible_child_name("disconnected")

        self.mainstack.connect('notify::visible-child', self.on_page_changed)

        SwitcherPage.__init__(self, builder)
        MediaPage.__init__(self, builder)
        AudioPage.__init__(self, builder)
        CameraPage.__init__(self, builder)

        self.status_model = builder.get_object('status_model')
        self.status_mode = builder.get_object('status_mode')
        self.focus_dummy = builder.get_object('focus_dummy')
        self.disable_shortcuts = False
        self.macro_edit = False

        self.timecode_mode = 0
        self.timecode_offset = 0

        self.aux_follow_audio = set()

        self.hardware_threads = {}
        self.connection_settings = {}
        self.inpr_latch = 0

        self.apply_css(self.window, self.provider)

        self.window.show_all()

        self.firmware_version = None
        self.mode = None

        self.connection = AtemConnection(self.on_change, self.on_disconnect, self.on_transfer_progress,
                                         self.on_download_done, self.on_connect, self.on_upload_done,
                                         self.on_upload_progress)
        self.routing = Routing(self.connection)

        if args.ip:
            self.connection.ip = args.ip
            if args.persist:
                self.settings.set_string('switcher-ip', args.ip)
        else:
            if self.connection.ip == "0.0.0.0":
                ConnectionWindow(self.window, self.connection, self.application)

            self.connection.ip = self.settings.get_string('switcher-ip')
        self.connection.daemon = True
        self.connection.start()

        accel = Gtk.AccelGroup()
        accel.connect(Gdk.keyval_from_name('space'), 0, 0, self.on_cut_shortcut)
        accel.connect(Gdk.keyval_from_name('ISO_Enter'), 0, 0, self.on_auto_shortcut)
        accel.connect(Gdk.keyval_from_name('Return'), 0, 0, self.on_auto_shortcut)
        accel.connect(Gdk.keyval_from_name('KP_Enter'), 0, 0, self.on_auto_shortcut)
        accel.connect(Gdk.keyval_from_name('F12'), 0, 0, self.on_debugger_shortcut)
        ctrl = Gdk.ModifierType.CONTROL_MASK
        accel.connect(Gdk.keyval_from_name('question'), ctrl, 0, self.on_help_shortcut)

        for i in range(0, 9):
            accel.connect(Gdk.keyval_from_name(str(i)), 0, 0, self.on_preview_keyboard_change)
            accel.connect(Gdk.keyval_from_name(str(i)), Gdk.ModifierType.CONTROL_MASK, 0,
                          self.on_program_keyboard_change)
            accel.connect(Gdk.keyval_from_name(str(i)), Gdk.ModifierType.MOD1_MASK, 0,
                          self.on_cutbus_keyboard_change)

        self.window.add_accel_group(accel)

        GLib.timeout_add_seconds(1, self.on_clock)

        Gtk.main()

    def on_preview_keyboard_change(self, widget, window, key, modifier):
        if self.disable_shortcuts:
            return

        index = key - 49
        cmd = PreviewInputCommand(index=0, source=index + 1)
        self.connection.mixer.send_commands([cmd])

    def on_program_keyboard_change(self, widget, window, key, modifier):
        if self.disable_shortcuts:
            return

        index = key - 49
        cmd = ProgramInputCommand(index=0, source=index + 1)
        self.connection.mixer.send_commands([cmd])

    def on_cutbus_keyboard_change(self, widget, window, key, modifier):
        if self.disable_shortcuts:
            return

        index = key - 49
        cmd = PreviewInputCommand(index=0, source=index + 1)
        auto = AutoCommand(index=0)
        self.connection.mixer.send_commands([cmd, auto])

    def on_entry_focus(self, *args):
        self.disable_shortcuts = True

    def on_entry_unfocus(self, *args):
        self.disable_shortcuts = False
        self.focus_dummy.grab_focus()

    def on_context_menu(self, *args):
        pass

    def on_entry_activate(self, *args):
        self.focus_dummy.grab_focus()

    def hook_up_focus(self, widget):
        widget.connect("focus-in-event", self.on_entry_focus)
        widget.connect("focus-out-event", self.on_entry_unfocus)
        widget.connect("activate", self.on_entry_activate)

    def apply_css(self, widget, provider):
        Gtk.StyleContext.add_provider(widget.get_style_context(),
                                      provider,
                                      Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)

        if isinstance(widget, Gtk.Container):
            widget.forall(self.apply_css, provider)

    def on_switcher_ip_changed(self, *args):
        self.log_aw.info("Connection settings changed")
        self.log_aw.info("Closing old connection...")
        self.connection.die()
        self.connection.join(timeout=1)
        for tid in self.hardware_threads:
            self.hardware_threads[tid].die()
        for me in self.me:
            self.me.remove(me)
        for widget in self.main_blocks:
            self.main_blocks.remove(widget)
        time.sleep(0.1)
        self.clear_audio_state()
        self.log_aw.info("Starting new connection to {}".format(self.settings.get_string('switcher-ip')))
        self.connection = AtemConnection(self.on_change, self.on_disconnect, self.on_transfer_progress,
                                         self.on_download_done, self.on_connect, self.on_upload_done,
                                         self.on_upload_progress)
        self.connection.daemon = True
        self.connection.ip = self.settings.get_string('switcher-ip')
        self.connection.start()

    def on_reconnect_clicked(self, widget, *args):
        self.log_aw.info("Reconnect clicked")
        self.on_switcher_ip_changed()

    def on_connection_settings_changed(self, *args):
        self.log_aw.info("Connection stored config changed")

    def set_class(self, widget, classname, state):
        if state:
            widget.get_style_context().add_class(classname)
        else:
            widget.get_style_context().remove_class(classname)

    def on_main_window_destroy(self, widget):
        Gtk.main_quit()

    def on_preferences_button_clicked(self, widget):
        ConnectionWindow(self.window, self.connection, self.application)

    def on_disconnect(self, reason):
        self.connectionstack.set_visible_child_name(reason)
        for tid in self.hardware_threads:
            self.hardware_threads[tid].die()
        self.log_aw.warning("Disconnected from mixer")

    def on_connect(self):
        self.inpr_latch = 0
        self.audio_latch = 0
        # Load stored connection-specific settings if available
        settings = self.settings.get_string('connections')
        settings = json.loads(settings)
        if self.connection.ip in settings:
            self.connection_settings = settings[self.connection.ip]
        else:
            self.connection_settings = {
                "videohubs": []
            }

        # Connect to stored videohubs
        if 'videohubs' in self.connection_settings:
            hubs = self.connection_settings['videohubs']
            for ip in hubs:
                hub = hubs[ip]
                thread = VideoHubConnection(ip, self.on_videohub_connect, self.on_videohub_disconnect,
                                            self.on_videohub_input_change, self.on_videohub_output_change,
                                            self.on_videohub_route_change)
                thread.daemon = True
                thread.id = f'hub-{ip}'
                thread.start()
                self.hardware_threads[thread.id] = thread

    def on_debugger_shortcut(self, *args):
        if self.disable_shortcuts:
            return

        DebuggerWindow(self.window, self.connection, self.application)

    def on_help_shortcut(self, *args):
        if self.disable_shortcuts:
            return

        builder = Gtk.Builder()
        builder.set_translation_domain("openswitcher")
        builder.add_from_resource('/nl/brixit/switcher/ui/shortcuts.ui')
        window = builder.get_object('shortcuts')
        window.set_transient_for(self.window)
        window.present()

    def on_videohub_connect(self, hub_id):
        self.log_aw.info(f"Connected to {hub_id}")
        from gtk_switcher.videohubbus import VideoHubBus
        for hubid in self.connection_settings['videohubs']:
            ip = self.connection_settings['videohubs'][hubid]['ip']
            for outputid in self.connection_settings['videohubs'][hubid]['outputs']:
                output = self.connection_settings['videohubs'][hubid]['outputs'][outputid]
                if output['bus']:
                    bus = VideoHubBus(self.provider, self.hardware_threads[f'hub-{ip}'], outputid)
                    self.main_blocks.add(bus)

            for me in self.me:
                if hasattr(me, 'add_router'):
                    me.add_router(self.hardware_threads[f'hub-{ip}'], self.connection_settings['videohubs'][hubid])

        self.main_blocks.show_all()

    def on_videohub_disconnect(self, hub_id):
        self.log_aw.warning(f"Disconnected from {hub_id}")

    def on_videohub_input_change(self, hub_id, index, inputs):
        pass

    def on_videohub_output_change(self, hub_id, index, outputs):
        pass

    def on_videohub_route_change(self, hub_id, index, source):
        hub = self.hardware_threads[hub_id]
        ip = hub.ip
        if 'videohubs' not in self.connection_settings:
            return

        if ip not in self.connection_settings['videohubs']:
            return

        if self.connection_settings['videohubs'][ip]['outputs'][str(index)]['rename']:
            atem_input = self.connection_settings['videohubs'][ip]['outputs'][str(index)]['source']
            new_name = hub.inputs[source]['label']
            if ':' in new_name:
                button, new_name = new_name.split(':', maxsplit=1)
                new_name = new_name.strip()
                button = button.strip()
            else:
                button = None

            cmd = InputPropertiesCommand(source_index=atem_input, label=new_name, short_label=button)
            self.connection.mixer.send_commands([cmd])

    def on_bypass_firmware_clicked(self, widget, *args):
        self.connectionstack.set_visible_child_name("connected")

    def on_change(self, field, data):
        global _callbacks
        if self.args.dump is not None and field in self.args.dump:
            if isinstance(data, bytes):
                print('== {} ({} bytes)=='.format(field, len(data)))
                hexdump(data)
            else:
                print('== {} ({} bytes)=='.format(field, len(data.raw)))
                hexdump(data.raw)
                print(data)

        # Call all the registered decorators
        call_fields(field, self, data)

        try:
            if self.inpr_latch == 1 and field != 'input-properties':
                # Run the input change event after the events on start have run
                self.inpr_latch = 2
                self.on_input_layout_change(None)
                self.on_camera_layout_change(None)

            if field == 'firmware-version':
                self.firmware_version = data
                if data.major < 2:
                    self.connectionstack.set_visible_child_name("firmware")
                elif data.minor < 28:
                    self.connectionstack.set_visible_child_name("firmware")
                else:
                    self.connectionstack.set_visible_child_name("connected")
                self.log_aw.info("Firmware: {}".format(data.version))
            elif field == 'input-properties':
                # Ignore input-properties fields in the initial connection
                if self.inpr_latch == 0:
                    self.inpr_latch = 1
                elif self.inpr_latch == 1:
                    return
                else:
                    self.on_input_layout_change(data)
                    self.on_camera_layout_change(data)
            elif field == 'product-name':
                self.status_model.set_text(data.name)
                self.log_aw.info("Mixer model: {}".format(data.name))
            elif field == 'video-mode':
                self.mode = data
                for me in self.me:
                    me.set_mode(data)
                self.status_mode.set_text(data.get_label())
            elif field == 'fairlight-master-properties':
                self.on_fairlight_master_properties_change(data)
            elif field == 'fairlight-audio-input':
                self.on_fairlight_audio_input_change(data)
            elif field == 'atem-eq-band-properties':
                self.on_fairlight_eq_band_change(data)
            elif field == 'audio-input':
                self.on_audio_input_change(data)
            elif field == 'fairlight-strip-properties':
                self.on_fairlight_strip_properties_change(data)
            elif field == 'fairlight-tally':
                self.on_fairlight_tally_change(data)
            elif field == 'audio-mixer-tally':
                self.on_audio_mixer_tally_change(data)
            elif field == 'audio-mixer-master-properties':
                self.on_audio_mixer_master_properties_change(data)
            elif field == 'audio-mixer-monitor-properties':
                self.on_audio_monitor_properties_change(data)
            elif field == 'aux-output-source':
                self.on_aux_output_source_change(data)
                if data.index in self.aux_follow_audio:
                    self.on_aux_monitor_source_change(data)
            elif field == 'audio-meter-levels':
                self.on_audio_meter_levels_change(data)
            elif field == 'fairlight-meter-levels':
                self.on_fairlight_meter_levels_change(data)
            elif field == 'fairlight-master-levels':
                self.on_fairlight_master_levels_change(data)

            else:
                if field == 'time':
                    return
                if not self.debug and self.args.dump is not None and len(self.args.dump) > 0:
                    return
                if isinstance(data, bytes):
                    self.log_aw.debug(field)
                else:
                    self.log_aw.debug(data)
        except Exception as e:
            # When the connection breaks on initial sync the UI events are queued but no mixer state is present
            # catch the exceptions and bring the software into the disconnected state cleanly
            if self.connection.connected:
                raise
            self.log_aw.error(f"Exception while disconnected: {e}")

    @field('time')
    def on_time_sync(self, data):
        seconds = data.total_seconds()
        if self.timecode_mode == 0:
            self.timecode_offset = seconds - datetime.now().timestamp()
        else:
            t = time.localtime()
            tod = 3600 * t.tm_hour + 60 * t.tm_min + t.tm_sec
            self.timecode_offset = tod - seconds

    @field('time-config')
    def on_timecode_config_change(self, data):
        self.timecode_mode = data.mode

    def on_clock(self):
        self.on_clock_stream_recorder()
        self.on_clock_stream_live()
        return True

    def on_transfer_progress(self, store, slot, progress):
        if store == 0:
            # Media transfer
            self.on_media_transfer_progress(slot, progress)

    def on_download_done(self, store, slot, data):
        if store == 0:
            # Media transfer
            self.on_media_download_done(slot, data)
        if store == 0xffff:
            # Macro fetch
            if self.macro_edit:
                self.macro_edit = False

                MacroEditorWindow(self.window, self.application, self.connection, slot, data)

    def on_upload_done(self, store, slot):
        if store == 0:
            self.on_media_upload_done(store, slot)

    def on_upload_progress(self, store, slot, percent, done, size):
        if store == 0:
            self.on_media_upload_progress(store, slot, percent, done, size)

    def on_page_changed(self, widget, *args):
        page = widget.get_visible_child_name()
        if page == 'media':
            self.on_page_media_open()
        if page == 'audio':
            self.enable_levels()
        else:
            self.disable_levels()

    @field('macro-record-status')
    def on_macro_record_status_changed(self, data):
        self.set_class(self.headerbar, 'recording', data.is_recording)
        if data.is_recording:
            self.macro_status_index.set_label(f"Recording macro {data.index}")
            self.header_context_stack.set_visible_child_name('macro')
        else:
            self.header_context_stack.set_visible_child_name('live')



# Copyright 2021 - 2022, Martijn Braam and the OpenAtem contributors
# SPDX-License-Identifier: GPL-3.0-only
import time
from urllib.parse import urlparse, quote

import gi

from gtk_switcher.adjustmententry import AdjustmentEntry
from gtk_switcher.dial import Dial
from gtk_switcher.eqcurve import EqCurve
from gtk_switcher.gtklogadjustment import LogAdjustment
from gtk_switcher.preferences import PreferencesWindow
from pyatem import locate
from pyatem.command import FairlightStripPropertiesCommand, FairlightEqBandPropertiesCommand
from pyatem.field import FairlightStripPropertiesField

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GLib, GObject, Gio, GdkPixbuf

gi.require_version('Handy', '1')
from gi.repository import Handy


class EqWindow:
    def __init__(self, strip_id, parent, connection, provider):
        self.strip_id = strip_id
        self.connection = connection
        self.provider = provider
        self.model_changing = True
        builder = Gtk.Builder()
        builder.set_translation_domain("openswitcher")
        builder.add_from_resource('/nl/brixit/switcher/ui/eq.glade')
        builder.connect_signals(self)

        window = builder.get_object("window")
        curvebox = builder.get_object("curvebox")
        self.eq_enable = builder.get_object('eq_enable')

        window.set_transient_for(parent)
        window.set_modal(True)

        self.curve = EqCurve()
        self.curve.mini = False
        curvebox.add(self.curve)

        strip = self.connection.mixer.mixerstate['fairlight-strip-properties'][self.strip_id]
        self.source = strip.index
        self.channel = strip.subchannel

        self.eq_enable.set_state(strip.eq_enable)
        self.curve.set_enabled(strip.eq_enable)

        self.box = []
        self.switch = []
        for i in range(0, 6):
            self.box.append(builder.get_object(f"controls_band{i + 1}"))

        self.adj_frequency = []
        self.adj_gain = []
        self.adj_q = []
        self.range = {}

        for i in range(0, 6):
            blocks = self.create_band(i, i == 0, i == 5)
            container = builder.get_object(f'controls_band{i + 1}')
            for b in blocks:
                container.add(b)

        for band_id in self.connection.mixer.mixerstate['atem-eq-band-properties'][self.strip_id]:
            band = self.connection.mixer.mixerstate['atem-eq-band-properties'][self.strip_id][band_id]
            self.on_band_change(band)

            switch = Gtk.Switch()
            self.switch.append(switch)

        self.model_changing = False

        self.apply_css(window, self.provider)

        self.connection.mixer.on(f'change:atem-eq-band-properties:{self.strip_id}', self.on_band_change)
        self.connection.mixer.on(f'change:fairlight-strip-properties:{self.strip_id}', self.on_strip_change)
        window.show_all()

    def create_control(self, name, adjustment, min, max):
        result = Gtk.Grid()
        label = Gtk.Label(label=name)
        label.get_style_context().add_class('dim-label')
        result.attach(label, 0, 0, 1, 1)

        entry = AdjustmentEntry(adjustment, min, max)
        entry.get_style_context().add_class('mini')
        entry.set_margin_left(16)
        entry.set_margin_right(16)
        entry.set_margin_end(8)
        entry.set_max_width_chars(6)
        entry.set_width_chars(6)
        entry.set_alignment(xalign=0.5)

        result.attach(entry, 0, 1, 1, 1)

        dial = Dial()
        dial.set_adjustment(adjustment)
        result.attach(dial, 1, 0, 1, 2)
        s = Gtk.Separator()
        result.attach(s, 0, 2, 2, 1)
        return result

    def create_band(self, index, first=False, last=False):
        lower = 30
        upper = 21700
        if first:
            upper = 395
        elif last:
            lower = 1400
        adj_frequency = Gtk.Adjustment(0, lower, upper, 10, 10, 0)
        adj_frequency.index = index
        adj_frequency.connect('value-changed', self.on_frequency_change)
        self.adj_frequency.append(adj_frequency)

        adj_gain = Gtk.Adjustment(0, -2000, 2000, 10, 10, 0)
        adj_gain.index = index
        adj_gain.connect('value-changed', self.on_gain_change)
        self.adj_gain.append(adj_gain)

        adj_q = Gtk.Adjustment(0, 30, 1030, 10, 10, 0)
        adj_q.index = index
        adj_q.connect('value-changed', self.on_q_change)
        self.adj_q.append(adj_q)

        blocks = []

        blocks.append(self.create_control('Frequency', adj_frequency, lower, upper))
        blocks.append(self.create_control('Gain', adj_gain, -20, 20))
        blocks.append(self.create_control('Q', adj_q, 0.3, 10.3))
        return blocks

    def apply_css(self, widget, provider):
        Gtk.StyleContext.add_provider(widget.get_style_context(),
                                      provider,
                                      Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)

        if isinstance(widget, Gtk.Container):
            widget.forall(self.apply_css, provider)

    def on_band_change(self, band):
        self.model_changing = True
        self.curve.update_band(band)

        self.adj_frequency[band.band_index].set_value(band.band_frequency)
        self.adj_gain[band.band_index].set_value(band.band_gain)
        self.adj_q[band.band_index].set_value(band.band_q)
        self.range[band.band_index] = band.band_freq_range
        self.model_changing = False

    def on_strip_change(self, strip):
        if not isinstance(strip, FairlightStripPropertiesField):
            return
        self.eq_enable.set_state(strip.eq_enable)
        self.curve.set_enabled(strip.eq_enable)

    def on_eq_enable_state_set(self, widget, enabled):
        if self.model_changing:
            return
        cmd = FairlightStripPropertiesCommand(source=self.source, channel=self.channel, eq_enable=enabled)
        self.connection.mixer.send_commands([cmd])

    def on_frequency_change(self, widget):
        if self.model_changing:
            return
        band = widget.index
        f = int(widget.get_value())
        range = self.range[band]
        old_range = range
        ranges = {
            1: [30, 395],
            2: [100, 1480],
            4: [450, 7910],
            8: [1400, 21700],
        }
        while True:
            if f < ranges[range][0]:
                range = int(range / 2)
                continue
            if f > ranges[range][1]:
                range = int(range * 2)
                continue
            break

        if range == old_range:
            range = None
        cmd = FairlightEqBandPropertiesCommand(source=self.source, channel=self.channel, band=band,
                                               frequency=f, band_range=range)
        self.connection.mixer.send_commands([cmd])

    def on_gain_change(self, widget):
        if self.model_changing:
            return
        cmd = FairlightEqBandPropertiesCommand(source=self.source, channel=self.channel, band=widget.index,
                                               gain=int(widget.get_value()))
        self.connection.mixer.send_commands([cmd])

    def on_q_change(self, widget):
        if self.model_changing:
            return
        cmd = FairlightEqBandPropertiesCommand(source=self.source, channel=self.channel, band=widget.index,
                                               q=int(widget.get_value()))
        self.connection.mixer.send_commands([cmd])



from pyatem.command import AuxSourceCommand
from pyatem.field import InputPropertiesField, AuxOutputSourceField


class Routing:
    def __init__(self, connection):
        self.connection = connection
        self.source = {}
        self.output = {}
        self.aux_map = {}

    def reset(self):
        pass

    def add_output(self, output: InputPropertiesField):
        if output.index in self.output:
            return

        self.output[output.index] = output

        self.aux_map = {}
        for i, port in enumerate(sorted(self.output)):
            self.aux_map[port] = i

    def aux_changed(self, aux: AuxOutputSourceField):
        pass

    def port_index_to_aux_index(self, port):
        return self.aux_map[port]

    def aux_index_to_port_index(self, aux):
        for a in self.aux_map:
            if self.aux_map[a] == aux:
                return a

    def change(self, destination, source):
        aux_idx = self.aux_map[destination]
        cmd = AuxSourceCommand(aux_idx, source=source)
        self.connection.mixer.send_commands([cmd])



# Copyright 2025, Martijn Braam and the OpenAtem contributors
# SPDX-License-Identifier: GPL-3.0-only
from gi.repository import Gtk, GObject, Gdk

from gtk_switcher.palettepanel import PalettePanel
from pyatem.command import SupersourceBoxPropertiesCommand, SupersourcePropertiesCommand
from pyatem.field import SupersourceBoxPropertiesField, SupersourcePropertiesField


class SupersourcePanel(PalettePanel):
    __gtype_name__ = 'SupersourcePanel'

    def __init__(self, connection, index, box_model, art_model):
        self.connection = connection
        self.name = _("SuperSource {}".format(index + 1))
        self.index = index

        super().__init__(self.name, connection, preset_domain="supersource",
                         preset_fields=['supersource-properties', 'supersource-box-properties'],
                         preset_override=[self.index])

        self.ctrl = {}

        tabs = self.add_tabs()
        for i in range(4):
            self.ctrl[i] = {}
            box = tabs.add_page(f"box{i}", _("Box {}").format(i + 1))
            self.ctrl[i]["enable"] = box.add_control_toggle(_("Enable box"), _("Enable"),
                                                            handler=self.on_box_enable, handler_args=[i])
            self.ctrl[i]["source"] = box.add_control_dropdown(_("Source"), box_model,
                                                              handler=self.on_box_source, handler_args=[i])
            self.ctrl[i]["adj_x"] = Gtk.Adjustment(lower=-4800, upper=4800)
            self.ctrl[i]["x"] = box.add_control_slider_box(_("Position X"), self.ctrl[i]["adj_x"],
                                                           display_min=-48, display_max=48,
                                                           handler=self.on_box_pos, handler_args=[i, "x"])
            self.ctrl[i]["adj_y"] = Gtk.Adjustment(lower=-3400, upper=3400)
            self.ctrl[i]["y"] = box.add_control_slider_box(_("Position Y"), self.ctrl[i]["adj_y"],
                                                           display_min=-34, display_max=34,
                                                           handler=self.on_box_pos, handler_args=[i, "y"])
            self.ctrl[i]["adj_size"] = Gtk.Adjustment(lower=7, upper=1000)
            self.ctrl[i]["size"] = box.add_control_slider_box(_("Size"), self.ctrl[i]["adj_size"],
                                                              display_min=0.07, display_max=1,
                                                              handler=self.on_box_pos, handler_args=[i, "size"])
            box.add_separator()

            self.ctrl[i]["crop"] = box.add_control_toggle(_("Crop enable"), _("Crop"),
                                                          handler=self.on_box_crop, handler_args=[i])
            self.ctrl[i]["adj_top"] = Gtk.Adjustment(lower=0, upper=18000)
            self.ctrl[i]["top"] = box.add_control_slider_box(_("Top"), self.ctrl[i]["adj_top"],
                                                             display_min=0, display_max=18,
                                                             handler=self.on_box_pos, handler_args=[i, "top"])
            self.ctrl[i]["adj_bottom"] = Gtk.Adjustment(lower=0, upper=18000)
            self.ctrl[i]["bottom"] = box.add_control_slider_box(_("Bottom"), self.ctrl[i]["adj_bottom"],
                                                                display_min=0, display_max=18,
                                                                handler=self.on_box_pos, handler_args=[i, "bottom"])
            self.ctrl[i]["adj_left"] = Gtk.Adjustment(lower=0, upper=32000)
            self.ctrl[i]["left"] = box.add_control_slider_box(_("Left"), self.ctrl[i]["adj_left"],
                                                              display_min=0, display_max=32,
                                                              handler=self.on_box_pos, handler_args=[i, "left"])
            self.ctrl[i]["adj_right"] = Gtk.Adjustment(lower=0, upper=32000)
            self.ctrl[i]["right"] = box.add_control_slider_box(_("Right"), self.ctrl[i]["adj_right"],
                                                               display_min=0, display_max=32,
                                                               handler=self.on_box_pos, handler_args=[i, "right"])

        art = tabs.add_page("art", _("Art"))
        self.art_fill = art.add_control_dropdown(_("Source"), art_model, handler=self.on_art_source,
                                                 handler_args=['fill_source'])
        self.art_key = art.add_control_dropdown(_("Key"), art_model, handler=self.on_art_source,
                                                handler_args=['key_source'])
        self.layer = art.add_control_radio(_("Layer"), {
            0: _("Background"),
            1: _("Foreground"),
        }, handler=self.on_layer)
        art.add_separator()
        self.adj_clip = Gtk.Adjustment(upper=1000, step_increment=1, page_increment=10)
        self.adj_gain = Gtk.Adjustment(upper=1000, step_increment=1, page_increment=10)
        self.premultiplied = art.add_control_toggle(_("Key"), _("Pre multiplied"), handler=self.on_premultiplied)
        self.clip = art.add_control_slider(_("Clip"), self.adj_clip, handler=self.on_clip)
        self.gain = art.add_control_slider(_("Gain"), self.adj_gain, handler=self.on_gain)
        self.invert = art.add_control_toggle(_("Invert"), _("Invert"), handler=self.on_invert)

        self.show_all()

    def __repr__(self):
        return '<SupersourcePanel {}>'.format(self.panel_name)

    @PalettePanel.event('change:supersource-properties:*')
    def event_properties(self, data: SupersourcePropertiesField):
        if data.index != self.index:
            return
        self.art_fill.set_active_id(str(data.fill_source))
        self.art_key.set_active_id(str(data.key_source))
        self.set_class(self.premultiplied, 'active', data.premultiplied)
        self.set_class(self.invert, 'active', data.inverted)

        self.layer.set_value(data.layer)

        self.clip.set_sensitive(not data.premultiplied)
        self.gain.set_sensitive(not data.premultiplied)
        self.clip.set_value(data.clip)
        self.gain.set_value(data.gain)

    @PalettePanel.event('change:supersource-box-properties:*')
    def event_boxproperties(self, data: SupersourceBoxPropertiesField):
        if data.index != self.index:
            return
        self.set_class(self.ctrl[data.box]["enable"], "active", data.enabled)
        self.ctrl[data.box]["source"].set_active_id(str(data.source))
        self.ctrl[data.box]["adj_x"].set_value(data.x)
        self.ctrl[data.box]["adj_y"].set_value(data.y)
        self.ctrl[data.box]["adj_size"].set_value(data.size)

        self.set_class(self.ctrl[data.box]["crop"], "active", data.masked)
        self.ctrl[data.box]["top"].set_value(data.mask_top)
        self.ctrl[data.box]["bottom"].set_value(data.mask_bottom)
        self.ctrl[data.box]["left"].set_value(data.mask_left)
        self.ctrl[data.box]["right"].set_value(data.mask_right)

    def on_box_enable(self, widget, box):
        state = widget.get_style_context().has_class('active')
        self.run(SupersourceBoxPropertiesCommand(self.index, box, enabled=not state))

    def on_box_source(self, widget, box):
        source = int(widget.get_active_id())
        self.run(SupersourceBoxPropertiesCommand(self.index, box, source=source))

    def on_box_crop(self, widget, box):
        state = widget.get_style_context().has_class('active')
        self.run(SupersourceBoxPropertiesCommand(self.index, box, masked=not state))

    def on_box_pos(self, widget, box, prop):
        kwarg = {prop: int(widget.get_value())}
        self.run(SupersourceBoxPropertiesCommand(self.index, box, **kwarg))

    def on_art_source(self, widget, prop):
        kwarg = {prop: int(widget.get_active_id())}
        self.run(SupersourcePropertiesCommand(self.index, **kwarg))

    def on_premultiplied(self, widget):
        state = widget.get_style_context().has_class('active')
        self.run(SupersourcePropertiesCommand(index=self.index, premultiplied=not state))

    def on_clip(self, widget):
        self.run(SupersourcePropertiesCommand(index=self.index, clip=int(widget.get_value())))

    def on_gain(self, widget):
        self.run(SupersourcePropertiesCommand(index=self.index, gain=int(widget.get_value())))

    def on_invert(self, widget):
        state = widget.get_style_context().has_class('active')
        self.run(SupersourcePropertiesCommand(index=self.index, invert=not state))

    def on_layer(self, widget, value):
        self.run(SupersourcePropertiesCommand(index=self.index, layer=value))



# Copyright 2021 - 2022, Martijn Braam and the OpenAtem contributors
# SPDX-License-Identifier: GPL-3.0-only
import math

from gi.repository import Gtk, GObject, Gdk


class BiQuad:
    def __init__(self):
        self.b0 = 1
        self.b1 = 1
        self.b2 = 1
        self.a0 = 1
        self.a1 = 1
        self.a2 = 1

    def normalize(self):
        self.b0 /= self.a0
        self.b1 /= self.a0
        self.b2 /= self.a0
        self.a1 /= self.a0
        self.a2 /= self.a0
        self.a0 = 1.0

    def get_coeff(self):
        return ([self.b0, self.b1, self.b2], [self.a0, self.a1, self.a2])

    def calculate(self, f):
        # Don't worry about it
        phi = (math.sin(math.pi * f * 2 / (2 * 48000))) ** 2
        r = ((self.b0 + self.b1 + self.b2) ** 2 - \
             4 * (self.b0 * self.b1 + 4 * self.b0 * self.b2 + \
                  self.b1 * self.b2) * phi + 16 * self.b0 * self.b2 * phi * phi) / \
            ((1 + self.a1 + self.a2) ** 2 - 4 * (self.a1 + 4 * self.a2 + \
                                                 self.a1 * self.a2) * phi + 16 * self.a2 * phi * phi)
        if r < 0:
            r = 0
        r = r ** (.5)
        try:
            return 20 * math.log10(r)
        except:
            return -200


class HighPass(BiQuad):
    def __init__(self, frequency):
        w0 = (2 * math.pi * frequency) / 48000
        s0 = math.sin(w0)
        c0 = math.cos(w0)
        q = 0.7071
        alpha = s0 / (2 * q)
        self.b0 = (1 + c0) / 2
        self.b1 = -(1 + c0)
        self.b2 = (1 + c0) / 2
        self.a0 = 1 + alpha
        self.a1 = -2 * c0
        self.a2 = 1 - alpha
        self.normalize()


class LowPass(BiQuad):
    def __init__(self, frequency):
        w0 = (2 * math.pi * frequency) / 48000
        s0 = math.sin(w0)
        c0 = math.cos(w0)
        q = 0.7071
        alpha = s0 / (2 * q)
        self.b0 = (1 - c0) / 2
        self.b1 = 1 - c0
        self.b2 = (1 - c0) / 2
        self.a0 = 1 + alpha
        self.a1 = -2 * c0
        self.a2 = 1 - alpha
        self.normalize()


class Peaking(BiQuad):
    def __init__(self, frequency, gain, q):
        A = math.pow(10, gain / 40)
        w0 = (2 * math.pi * frequency) / 48000
        if q == 0:
            q = 1
        alpha = math.sin(w0) / (2 * q)
        self.b0 = 1 + alpha * A
        self.b1 = -2 * math.cos(w0)
        self.b2 = 1 - alpha * A
        self.a0 = 1 + alpha / A
        self.a1 = -2 * math.cos(w0)
        self.a2 = 1 - alpha / A
        self.normalize()


class Notch(BiQuad):
    def __init__(self, frequency):
        w0 = (2 * math.pi * frequency) / 48000
        q = 7
        alpha = math.sin(w0) / (2 * q)
        self.b0 = 1
        self.b1 = -2 * math.cos(w0)
        self.b2 = 1
        self.a0 = 1 + alpha
        self.a1 = -2 * math.cos(w0)
        self.a2 = 1 - alpha
        self.normalize()


class HighShelf(BiQuad):
    def __init__(self, frequency, gain):
        A = math.pow(10, gain / 40)
        w0 = (2 * math.pi * frequency) / 48000
        s0 = math.sin(w0)
        c0 = math.cos(w0)
        beta = math.sqrt(A + A)
        self.b0 = A * ((A + 1) + (A - 1) * c0 + beta * s0)
        self.b1 = -2 * A * ((A - 1) + (A + 1) * c0)
        self.b2 = A * ((A + 1) + (A - 1) * c0 - beta * s0)
        self.a0 = (A + 1) - (A - 1) * c0 + beta * s0
        self.a1 = 2 * ((A - 1) - (A + 1) * c0)
        self.a2 = (A + 1) - (A - 1) * c0 - beta * s0
        self.normalize()


class LowShelf(BiQuad):
    def __init__(self, frequency, gain):
        A = math.pow(10, gain / 40)
        w0 = (2 * math.pi * frequency) / 48000
        s0 = math.sin(w0)
        c0 = math.cos(w0)
        beta = math.sqrt(A + A)
        self.b0 = A * ((A + 1) - (A - 1) * c0 + beta * s0)
        self.b1 = 2 * A * ((A - 1) - (A + 1) * c0)
        self.b2 = A * ((A + 1) - (A - 1) * c0 - beta * s0)
        self.a0 = (A + 1) + (A - 1) * c0 + beta * s0
        self.a1 = -2 * ((A - 1) + (A + 1) * c0)
        self.a2 = (A + 1) + (A - 1) * c0 - beta * s0
        self.normalize()


class EqCurve(Gtk.Frame):
    __gtype_name__ = 'EqCurve'

    def __init__(self):
        super(Gtk.Frame, self).__init__()
        self.connection = None
        self.set_size_request(900, 240)

        self.da = Gtk.DrawingArea()
        self.add(self.da)

        self.da.set_events(
            self.da.get_events()
            | Gdk.EventMask.LEAVE_NOTIFY_MASK
            | Gdk.EventMask.BUTTON_PRESS_MASK
            | Gdk.EventMask.BUTTON_RELEASE_MASK
            | Gdk.EventMask.POINTER_MOTION_MASK
            | Gdk.EventMask.POINTER_MOTION_HINT_MASK
        )

        self.da.connect("draw", self.on_draw)
        # self.da.connect("button-press-event", self.on_mouse_down)
        self.da.connect("button-release-event", self.on_mouse_up)
        # self.da.connect("motion-notify-event", self.on_mouse_move)
        self.show_all()
        self.bands = {}
        self.mini = True
        self.enabled = True

    def update_band(self, bandupdate):
        self.bands[bandupdate.band_index] = bandupdate
        self.da.queue_draw()

    def set_enabled(self, enabled):
        self.enabled = enabled
        self.da.queue_draw()

    def _x_to_f(self, width, x):
        frac = (x + 1) / (width + 1)

        log_min = math.log(19, 2)
        log_max = math.log(19000, 2)
        log_delta = log_max - log_min
        return 2 ** (log_min + (log_delta * frac))

    def _f_to_x(self, width, f):
        start = 0
        end = width
        while True:
            middle = start + ((end - start) / 2)
            pf = self._x_to_f(width, middle)
            distance = abs(pf - f)
            if distance < 3 or abs(end - start) < 2:
                break
            if pf < f:
                start = middle
            else:
                end = middle
        return middle

    def _x_to_f_aligned(self, width, x):
        """
        This is hack to shift the sampling frequency slightly so it corresponds to the peak of the nearest filter
        if there is a filter in the exact range covered by the pixel. This fixes aliasing in the display for very
        steep filters and is a lot more efficient than oversampling the whole frequency range.
        """
        lower = self._x_to_f(width, max(0, x - 1))
        upper = self._x_to_f(width, min(width, x + 1))
        for band_idx in self.bands:
            if not self.bands[band_idx].band_enabled:
                continue
            if self.bands[band_idx].band_filter not in [0x04, 0x08]:
                continue
            if self.bands[band_idx].band_gain == 0:
                continue
            bf = self.bands[band_idx].band_frequency
            if lower <= bf <= upper:
                return bf
        return self._x_to_f(width, x)

    def _y_to_db(self, height, y):
        fract = (y / height) * 2 - 1
        return height - (fract * 20)

    def _db_to_y(self, height, db):
        fract = db / 20
        return height - (fract * (height / 2) + height / 2)

    def calculate_filter(self, f):
        biquads = []
        for band_index in self.bands:
            band = self.bands[band_index]
            if not band.band_enabled:
                continue

            if band.band_filter == 0x01:
                # Low shelf
                biquads.append(LowShelf(band.band_frequency, band.band_gain / 100))
            elif band.band_filter == 0x02:
                # Low pass
                biquads.append(LowPass(band.band_frequency))
            elif band.band_filter == 0x04:
                # Bell
                biquads.append(Peaking(band.band_frequency, band.band_gain / 100, band.band_q / 100))
            elif band.band_filter == 0x08:
                # Notch
                biquads.append(Notch(band.band_frequency))
            elif band.band_filter == 0x10:
                # High pass band
                biquads.append(HighPass(band.band_frequency))
            elif band.band_filter == 0x20:
                # High shelf
                biquads.append(HighShelf(band.band_frequency, band.band_gain / 100))

        db = 0.0
        for band in biquads:
            db += band.calculate(f)
        return db

    def on_draw(self, widget, cr):
        width, height = widget.get_allocated_width(), widget.get_allocated_height()
        context = self.get_style_context()

        context.save()
        context.add_class('eq-window')
        if self.mini:
            context.add_class('mini')
        Gtk.render_background(context, cr, 0, 0, width, height)

        context.restore()
        context.save()
        context.add_class('eq-window-lines')
        context.add_class('zero')
        Gtk.render_line(context, cr, 0, height / 2, width, height / 2)

        if not self.mini:
            context.remove_class('zero')
            for dB in [15, 10, 5, -5, -10, -15]:
                Gtk.render_line(context, cr, 0, self._db_to_y(height, dB), width, self._db_to_y(height, dB))
            for Hz in [31, 62, 125, 250, 500, 1000, 2000, 4000, 8000, 16000]:
                x = self._f_to_x(width, Hz)
                Gtk.render_line(context, cr, x, 0, x, height)

        context.restore()
        context.save()
        context.add_class('eq-window-curve')
        if not self.enabled:
            context.add_class('disabled')

        step = 1 if self.mini else 3

        last = self._db_to_y(height, self.calculate_filter(19))
        for i in range(0, width, step):
            f = self._x_to_f_aligned(width, i)
            gain = self.calculate_filter(f)
            y = self._db_to_y(height, gain)
            Gtk.render_line(context, cr, i - step, last, i, y)
            last = y

        if not self.mini:
            context.restore()
            context.save()
            context.add_class('eq-window-handle')
            if not self.enabled:
                context.add_class('disabled')
            for band_index in self.bands:
                band = self.bands[band_index]
                if band.band_enabled:
                    context.add_class('active')
                else:
                    context.remove_class('active')
                x = self._f_to_x(width, band.band_frequency)
                y = self._db_to_y(height, band.band_gain / 100)
                Gtk.render_background(context, cr, x - 8, y - 8, 16, 16)
                Gtk.render_frame(context, cr, x - 8, y - 8, 16, 16)
                cr.move_to(x - 3, y + 4)
                color = context.get_color(Gtk.StateFlags.NORMAL)
                cr.set_source_rgba(color.red, color.green, color.blue, color.alpha)
                cr.show_text(f"{band_index + 1}")

        context.restore()

    def on_mouse_up(self, widget, *args):
        if self.mini:
            pass



import argparse
import threading
from http.server import HTTPServer, BaseHTTPRequestHandler
from urllib.parse import urlparse, parse_qsl

import pyatem.command as commandmodule
from pyatem.protocol import AtemProtocol

switcher = None
_port = None
_inputs = {}


class ApiServer(BaseHTTPRequestHandler):
    def do_GET(self):
        path = self.path
        print(path)

        if path == '/':
            self.send_response(200)
            self.send_header('Content-type', 'text/plain')
            self.end_headers()
            self.wfile.write(b"OpenSwitcher API server\n")
            return

        parts = urlparse(path)
        command = parts.path[1:]
        args = parts.query

        classname = command.title().replace('-', '') + "Command"
        print("Command: " + classname)
        if not hasattr(commandmodule, classname):
            self.send_response(404)
            self.send_header('Content-type', 'text/plain')
            self.end_headers()
            self.wfile.write(b"Unknown command\n")
            return

        arguments = dict(parse_qsl(args))
        for key in arguments:
            try:
                arguments[key] = int(arguments[key])
            except:
                pass
        if 'source' in arguments:
            if arguments['source'] in _inputs:
                arguments['source'] = _inputs[arguments['source']]

        try:
            cmd = getattr(commandmodule, classname)(**arguments)
            switcher.send_commands([cmd])
        except Exception as e:
            self.send_response(400)
            self.send_header('Content-type', 'text/plain')
            self.end_headers()
            self.wfile.write(str(e).encode())
            return
        self.send_response(200)
        self.send_header('Content-type', 'text/plain')
        self.end_headers()
        self.wfile.write(b"OK\n")


class WebThread(threading.Thread):
    def run(self):
        print("Running API server on :" + str(_port))
        httpd = HTTPServer(("", _port), ApiServer)
        httpd.serve_forever()


def input_changed(inputproperties):
    global _inputs
    _inputs[inputproperties.short_name] = inputproperties.index


def connection_ready(*args):
    WebThread().start()


def main():
    global switcher, _port
    parser = argparse.ArgumentParser(description="Atem CLI")
    parser.add_argument('ip', help='Atem ip or "usb" for usb')
    parser.add_argument('port', help='Port to run the HTTP server on', type=int)
    args = parser.parse_args()

    _port = args.port

    if args.ip == 'usb':
        print("Connecting to USB switcher")
        switcher = AtemProtocol(usb='auto')
    else:
        print("Connecting to " + args.ip)
        switcher = AtemProtocol(ip=args.ip)

    switcher.on('change:input-properties:*', input_changed)
    switcher.on('connected', connection_ready)

    switcher.connect()
    while True:
        switcher.loop()


if __name__ == '__main__':
    main()



# Copyright 2025, Martijn Braam and the OpenAtem contributors
# SPDX-License-Identifier: GPL-3.0-only
from gi.repository import Gtk, GObject, Gdk

from gtk_switcher.palettepanel import PalettePanel
from pyatem.command import ColorGeneratorCommand, DkeySetFillCommand, DkeySetKeyCommand, DkeyGainCommand, \
    DkeyMaskCommand
from pyatem.field import DkeyPropertiesField


class DownstreamKeyerPanel(PalettePanel):
    __gtype_name__ = 'DownstreamKeyerPanel'

    def __init__(self, connection, index, model_fill, model_key):
        self.connection = connection
        self.name = _("Downstream keyer {}").format(index + 1)
        self.index = index

        super().__init__(self.name, connection, preset_domain="dsk", preset_override=[self.index], preset_fields=[
            'dkey-properties-base',
            'dkey-properties'
        ])

        self.adj_clip = Gtk.Adjustment(upper=1000, step_increment=1, page_increment=10)
        self.adj_gain = Gtk.Adjustment(upper=1000, step_increment=1, page_increment=10)

        self.adj_top = Gtk.Adjustment(upper=9000, lower=-9000, step_increment=1, page_increment=10)
        self.adj_bottom = Gtk.Adjustment(upper=9000, lower=-9000, step_increment=1, page_increment=10)
        self.adj_left = Gtk.Adjustment(upper=16000, lower=-16000, step_increment=1, page_increment=10)
        self.adj_right = Gtk.Adjustment(upper=16000, lower=-16000, step_increment=1, page_increment=10)

        self.fill_source = self.add_control_dropdown(_("Fill source"), model_fill, handler=self.on_fill_change)
        self.key_source = self.add_control_dropdown(_("Key source"), model_key, handler=self.on_key_change)
        self.add_separator()
        self.premultiplied = self.add_control_toggle(_("Key"), _("Pre multiplied"), handler=self.on_premultiplied)
        self.clip = self.add_control_slider(_("Clip"), self.adj_clip, handler=self.on_clip)
        self.gain = self.add_control_slider(_("Gain"), self.adj_gain, handler=self.on_gain)
        self.invert = self.add_control_toggle(_("Invert"), _("Invert"), handler=self.on_invert)
        self.add_separator()
        self.mask_enable = self.add_control_toggle(_("Use mask"), _("Mask"), handler=self.on_mask_enable)

        self.top = self.add_control_slider_box(_("Top"), self.adj_top, display_min=-9, display_max=9,
                                               handler=self.on_mask, handler_args=['top'])
        self.bottom = self.add_control_slider_box(_("Bottom"), self.adj_bottom, display_min=-9, display_max=9,
                                                  handler=self.on_mask, handler_args=['bottom'])
        self.left = self.add_control_slider_box(_("Left"), self.adj_left, display_min=-16, display_max=16,
                                                handler=self.on_mask, handler_args=['left'])
        self.right = self.add_control_slider_box(_("Right"), self.adj_right, display_min=-16, display_max=16,
                                                 handler=self.on_mask, handler_args=['right'])

        self.show_all()

    def __repr__(self):
        return '<DownstreamKeyerPanel {}>'.format(self.panel_name)

    @PalettePanel.event('change:dkey-properties-base:*')
    def event_key_properties_base(self, data):
        if data.index != self.index:
            return

        with self.model:
            self.fill_source.set_active_id(str(data.fill_source))
            self.key_source.set_active_id(str(data.key_source))

    @PalettePanel.event('change:dkey-properties:*')
    def event_dkey_properties(self, data: DkeyPropertiesField):
        if data.index != self.index:
            return

        self.set_class(self.premultiplied, 'active', data.premultiplied)
        self.set_class(self.invert, 'active', data.invert_key)
        self.set_class(self.mask_enable, 'active', data.masked)

        self.clip.set_sensitive(not data.premultiplied)
        self.gain.set_sensitive(not data.premultiplied)
        self.adj_clip.set_value(data.clip)
        self.adj_gain.set_value(data.gain)

        self.adj_top.set_value(data.top)
        self.adj_bottom.set_value(data.bottom)
        self.adj_left.set_value(data.left)
        self.adj_right.set_value(data.right)

    def on_fill_change(self, widget):
        source = int(widget.get_active_id())
        self.run(DkeySetFillCommand(index=self.index, source=source))

    def on_key_change(self, widget):
        source = int(widget.get_active_id())
        self.run(DkeySetKeyCommand(index=self.index, source=source))

    def on_premultiplied(self, widget):
        state = widget.get_style_context().has_class('active')
        self.run(DkeyGainCommand(index=self.index, premultiplied=not state))

    def on_clip(self, widget):
        self.run(DkeyGainCommand(index=self.index, clip=int(widget.get_value())))

    def on_gain(self, widget):
        self.run(DkeyGainCommand(index=self.index, gain=int(widget.get_value())))

    def on_invert(self, widget):
        state = widget.get_style_context().has_class('active')
        self.run(DkeyGainCommand(index=self.index, invert=not state))

    def on_mask(self, widget, side):
        prop = {side: int(widget.get_value())}
        self.run(DkeyMaskCommand(index=self.index, **prop))

    def on_mask_enable(self, widget):
        state = widget.get_style_context().has_class('active')
        self.run(DkeyMaskCommand(index=self.index, enabled=not state))



# Copyright 2021 - 2022, Martijn Braam and the OpenAtem contributors
# SPDX-License-Identifier: GPL-3.0-only
import gi

from gtk_switcher.adjustmententry import AdjustmentEntry
from gtk_switcher.dial import Dial
from gtk_switcher.eqwindow import EqWindow
from gtk_switcher.gtklogadjustment import LogAdjustment
from gtk_switcher.eqcurve import EqCurve
from pyatem.command import AudioInputCommand, FairlightStripPropertiesCommand, FairlightMasterPropertiesCommand, \
    AudioMasterPropertiesCommand, AudioMonitorPropertiesCommand, SendAudioLevelsCommand, SendFairlightLevelsCommand, \
    AuxSourceCommand

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GLib, GObject, Gio, Gdk

gi.require_version('Handy', '1')
from gi.repository import Handy


class AudioPage:
    def __init__(self, builder):
        self.audio_channels = builder.get_object('audio_channels')

        self.mixer = 'atem'

        self.volume_level = {}
        self.input_gain = {}
        self.pan = {}
        self.delay = {}
        self.audio_tally = {}
        self.audio_strip = {}
        self.audio_on = {}
        self.audio_afv = {}
        self.audio_monitor = {}
        self.vu = {}
        self.eq = {}

        self.master_level = None
        self.master_afv = None
        self.monitor_level = None
        self.monitor_on = None
        self.monitor_dim = None

    def clear_audio_state(self):
        self.mixer = 'atem'
        self.volume_level = {}
        self.input_gain = {}
        self.pan = {}
        self.delay = {}
        self.audio_tally = {}
        self.audio_strip = {}
        self.audio_on = {}
        self.audio_afv = {}
        self.audio_monitor = {}
        self.vu = {}
        self.eq = {}

        self.master_level = None
        self.master_afv = None
        self.monitor_level = None
        self.monitor_on = None
        self.monitor_dim = None

        self.unrealize_audio()

    def _make_mixer_ui_label(self, index, name):
        label = Gtk.Label(label=name)
        label.get_style_context().add_class('dim-label')
        label.set_halign(Gtk.Align.END)
        label.set_valign(Gtk.Align.START)
        self.audio_channels.attach(label, 0, index, 1, 1)

    def unrealize_audio(self):
        for child in self.audio_channels:
            child.destroy()

    def make_mixer_ui(self, inputs):
        # Clear the existing channels
        self.unrealize_audio()

        # Create row of labels again
        if self.mixer == "fairlight":
            self._make_mixer_ui_label(2, "Input")
            self._make_mixer_ui_label(3, "Equalizer")
            self._make_mixer_ui_label(4, "Dynamics")
        self._make_mixer_ui_label(5, "dB")
        self._make_mixer_ui_label(6, "Pan")

        # Create channel strips
        strip_index = 1
        last_type = 0
        for input in inputs.values():
            if self.mixer == 'atem':
                self.audio_strip[input.strip_id] = input

            # Fairlight stereo inputs can be split in multiple mono strips
            num_subchannels = 1
            if self.mixer == 'fairlight' and input.split == 4:
                num_subchannels = 2

            # Create seperator if the last strip was a different category
            if input.type != last_type:
                last_type = input.type
                type_sep = Gtk.Separator(orientation=Gtk.Orientation.VERTICAL)
                type_sep.get_style_context().add_class('dark')
                type_sep.set_margin_left(8)
                type_sep.set_margin_right(8)
                self.audio_channels.attach(type_sep, strip_index, 0, 1, 8)
                strip_index += 1

            # Get channelstrip label
            if input.type == 0:
                label = self.connection.mixer.mixerstate['input-properties'][input.index].short_name
            elif input.type == 1:
                label = "MP {}".format(input.number + 1)
            else:
                connector = 'Analog'
                if hasattr(input, 'plug_name'):
                    connector = input.plug_name()
                elif 1300 <= input.index <= 1400:
                    connector = 'Mic'
                label = f"{connector} {input.number + 1}"

            label = Gtk.Label(label=label)
            self.audio_channels.attach(label, strip_index, 0, num_subchannels, 1)

            # Loop through all the strips generated by this input, always 1 for atem audio
            for c in range(0, num_subchannels):
                if self.mixer == 'fairlight':
                    strip_id = str(input.index) + '.' + str(c)
                else:
                    strip_id = input.strip_id

                # Create GTK backing controls if the strip has never existed
                if strip_id not in self.volume_level:
                    if self.mixer == 'atem':
                        self.volume_level[strip_id] = LogAdjustment(input.volume, 0, 65381, 10, 10, 100)
                        self.pan[strip_id] = Gtk.Adjustment(input.balance, -10000, 10000, 10, 10, 100)
                    else:
                        self.volume_level[strip_id] = LogAdjustment(0, -10000, 1000, 10, 10, 100, exp=True, range=30)
                        self.pan[strip_id] = Gtk.Adjustment(0, -10000, 10000, 10, 10, 100)
                        self.input_gain[strip_id] = LogAdjustment(0, -10000, 600, 10, 10, 100, exp=True, range=30)
                        self.delay[strip_id] = Gtk.Adjustment(0, 0, 8, 1, 1, 1)

                    self.volume_level[strip_id].connect('value-changed', self.on_volume_changed)
                    if strip_id in self.input_gain:
                        self.input_gain[strip_id].connect('value-changed', self.on_input_gain_changed)
                    self.pan[strip_id].connect('value-changed', self.on_pan_changed)

                    self.volume_level[strip_id].source = input.index
                    self.volume_level[strip_id].channel = c if num_subchannels > 1 else -1
                    self.pan[strip_id].source = input.index
                    self.pan[strip_id].channel = c if num_subchannels > 1 else -1
                    if self.mixer == 'fairlight':
                        self.input_gain[strip_id].source = input.index
                        self.input_gain[strip_id].channel = c if num_subchannels > 1 else -1

                # Create the tally "led"
                tally = Gtk.Box()
                tally.get_style_context().add_class('tally')
                if strip_id in self.audio_strip:
                    if self.mixer == 'fairlight':
                        tally_state = self.audio_strip[strip_id].state & 4
                    else:
                        tally_state = self.audio_strip[strip_id].state == 2
                    self.set_class(tally, 'afv', tally_state)
                self.audio_tally[strip_id] = tally
                self.audio_channels.attach(tally, strip_index + c, 1, 1, 1)

                # Create the controls
                if self.mixer == 'fairlight':
                    if len(self.input_gain) == 0:
                        return
                    input_frame = Gtk.Frame()
                    input_frame.get_style_context().add_class('view')
                    dial = Dial()
                    dial.set_adjustment(self.input_gain[strip_id])
                    gain_input = AdjustmentEntry(self.input_gain[strip_id], -100, 6)
                    gain_input.get_style_context().add_class('mini')
                    gain_input.set_margin_left(16)
                    gain_input.set_margin_right(16)
                    gain_input.set_margin_end(8)
                    gain_input.set_max_width_chars(6)
                    gain_input.set_width_chars(6)
                    gain_input.set_alignment(xalign=0.5)
                    self.hook_up_focus(gain_input)
                    gain_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
                    gain_box.add(dial)
                    gain_box.add(gain_input)
                    input_frame.add(gain_box)
                    self.audio_channels.attach(input_frame, strip_index + c, 2, 1, 1)

                    eq_frame = EqCurve()
                    eq_frame.connect('button-release-event', self.on_open_eq_overlay)
                    eq_frame.strip_id = strip_id
                    if 'atem-eq-band-properties' in self.connection.mixer.mixerstate and strip_id in \
                            self.connection.mixer.mixerstate['atem-eq-band-properties']:
                        for band_id in self.connection.mixer.mixerstate['atem-eq-band-properties'][strip_id]:
                            band = self.connection.mixer.mixerstate['atem-eq-band-properties'][strip_id][band_id]
                            eq_frame.update_band(band)
                    if 'fairlight-strip-properties' in self.connection.mixer.mixerstate and strip_id in \
                            self.connection.mixer.mixerstate['fairlight-strip-properties']:
                        strip = self.connection.mixer.mixerstate['fairlight-strip-properties'][strip_id]
                        eq_frame.set_enabled(strip.eq_enable)

                    eq_frame.get_style_context().add_class('view')
                    eq_frame.set_size_request(0, 64)
                    self.eq[strip_id] = eq_frame
                    self.audio_channels.attach(eq_frame, strip_index + c, 3, 1, 1)

                    dynamics_frame = Gtk.Frame()
                    dynamics_frame.get_style_context().add_class('view')
                    dynamics_frame.set_size_request(0, 64)
                    self.audio_channels.attach(dynamics_frame, strip_index + c, 4, 1, 1)

                # Volume fader
                volume_frame = Gtk.Frame()
                volume_frame.get_style_context().add_class('view')
                volume_frame.set_size_request(76, 0)
                volume_frame.set_vexpand(True)
                volume_box = Gtk.Box()
                volume_frame.add(volume_box)
                volume_slider = Gtk.Scale(orientation=Gtk.Orientation.VERTICAL,
                                          adjustment=self.volume_level[strip_id])
                volume_slider.set_inverted(True)
                volume_slider.set_draw_value(False)
                volume_slider.get_style_context().add_class('volume')
                volume_slider.connect('button-press-event', self.on_slider_held)
                volume_slider.connect('button-release-event', self.on_slider_released)
                volume_box.pack_start(volume_slider, True, True, 0)
                volume_box.set_margin_left(8)
                volume_box.set_margin_right(8)
                volume_box.set_margin_top(24)
                volume_box.set_margin_bottom(8)
                vu_left = Gtk.ProgressBar()
                vu_right = Gtk.ProgressBar()
                vu_left.set_orientation(Gtk.Orientation.VERTICAL)
                vu_left.set_inverted(True)
                vu_right.set_orientation(Gtk.Orientation.VERTICAL)
                vu_right.set_inverted(True)
                self.vu[strip_id] = (vu_left, vu_right)
                volume_box.pack_start(vu_left, False, True, 0)
                volume_box.pack_start(vu_right, False, True, 0)
                self.audio_channels.attach(volume_frame, strip_index + c, 5, 1, 1)

                # Pan dial
                pan_frame = Gtk.Frame()
                pan_frame.get_style_context().add_class('view')
                pan_dial = Dial()
                pan_dial.set_adjustment(self.pan[strip_id])

                pan_input = AdjustmentEntry(self.pan[strip_id], -100, 100)
                pan_input.get_style_context().add_class('mini')
                pan_input.set_margin_left(16)
                pan_input.set_margin_right(16)
                pan_input.set_margin_end(8)
                pan_input.set_max_width_chars(6)
                pan_input.set_width_chars(6)
                pan_input.set_alignment(xalign=0.5)
                self.hook_up_focus(pan_input)
                pan_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
                pan_box.add(pan_dial)
                pan_box.add(pan_input)
                pan_frame.add(pan_box)
                self.audio_channels.attach(pan_frame, strip_index + c, 6, 1, 1)

                # Channel strip routing controls
                routing_grid = Gtk.Grid()
                routing_grid.set_row_homogeneous(True)
                routing_grid.set_column_homogeneous(True)
                routing_grid.set_row_spacing(8)
                routing_grid.set_column_spacing(8)
                on = Gtk.Button(label="ON")
                on.source = input.index
                on.channel = c if num_subchannels > 1 else -1
                on.connect('clicked', self.do_channel_on)
                on.get_style_context().add_class('bmdbtn')
                routing_grid.attach(on, 0, 0, 1, 1)
                afv = Gtk.Button(label="AFV")
                afv.connect('clicked', self.do_channel_afv)
                afv.get_style_context().add_class('bmdbtn')
                afv.source = input.index
                afv.channel = c if num_subchannels > 1 else -1
                if input.type == 2:
                    afv.set_sensitive(False)
                routing_grid.attach(afv, 1, 0, 1, 1)
                mon = Gtk.Button(label="Monitor")
                mon.get_style_context().add_class('bmdbtn')
                mon.connect('clicked', self.do_channel_monitor)
                mon.source = input.index
                mon.channel = c if num_subchannels > 1 else -1
                routing_grid.attach(mon, 0, 1, 2, 1)
                self.audio_channels.attach(routing_grid, strip_index + c, 7, 1, 1)
                self.audio_on[strip_id] = on
                self.audio_afv[strip_id] = afv
                self.audio_monitor[strip_id] = mon
                if strip_id in self.audio_strip:
                    if self.mixer == 'atem':
                        self.set_class(afv, 'active', self.audio_strip[strip_id].state == 2)
                        self.set_class(on, 'program', self.audio_strip[strip_id].state == 1)
                    else:
                        self.set_class(afv, 'active', self.audio_strip[strip_id].state & 4)
                        self.set_class(on, 'program', self.audio_strip[strip_id].state & 2)

            strip_index += num_subchannels

        # Create the seperator for the master bus group
        master_sep = Gtk.Separator(orientation=Gtk.Orientation.VERTICAL)
        master_sep.get_style_context().add_class('dark')
        master_sep.set_margin_left(8)
        master_sep.set_margin_right(8)
        self.audio_channels.attach(master_sep, strip_index, 0, 1, 8)
        strip_index += 1

        # Create the bus strips
        for c in range(0, 1):
            label = Gtk.Label(label="Master")
            self.audio_channels.attach(label, strip_index + c, 0, 1, 1)
            tally = Gtk.Box()
            tally.get_style_context().add_class('tally')
            tally.get_style_context().add_class('program')
            self.audio_channels.attach(tally, strip_index + c, 1, 1, 1)

            volume_frame = Gtk.Frame()
            volume_frame.set_size_request(76, 0)
            volume_frame.get_style_context().add_class('view')
            volume_frame.set_vexpand(True)
            volume_box = Gtk.Box()
            volume_frame.add(volume_box)
            volume_slider = Gtk.Scale(orientation=Gtk.Orientation.VERTICAL,
                                      adjustment=self.master_level)
            volume_slider.set_inverted(True)
            volume_slider.set_draw_value(False)
            volume_slider.get_style_context().add_class('volume')
            volume_slider.connect('button-press-event', self.on_slider_held)
            volume_slider.connect('button-release-event', self.on_slider_released)
            volume_box.pack_start(volume_slider, True, True, 0)
            volume_box.set_margin_left(8)
            volume_box.set_margin_right(8)
            volume_box.set_margin_top(24)
            volume_box.set_margin_bottom(8)
            vu_left = Gtk.ProgressBar()
            vu_right = Gtk.ProgressBar()
            vu_left.set_orientation(Gtk.Orientation.VERTICAL)
            vu_left.set_inverted(True)
            vu_right.set_orientation(Gtk.Orientation.VERTICAL)
            vu_right.set_inverted(True)
            self.vu['master'] = (vu_left, vu_right)
            volume_box.pack_start(vu_left, False, True, 0)
            volume_box.pack_start(vu_right, False, True, 0)
            self.audio_channels.attach(volume_frame, strip_index + c, 5, 1, 1)

            # Create monitoring frame
            self.master_afv = Gtk.Button(label="AFV")
            self.master_afv.connect('clicked', self.do_master_afv)
            self.master_afv.set_margin_bottom(8)
            self.master_afv.get_style_context().add_class('bmdbtn')

            label = Gtk.Label("Monitor")
            monitoring_wrap = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
            monitoring_frame = Gtk.Frame()
            monitoring_frame.get_style_context().add_class('view')
            monitoring_wrap.add(self.master_afv)
            monitoring_wrap.add(monitoring_frame)
            self.audio_channels.attach(monitoring_wrap, strip_index + c, 6, 1, 2)
            monitor_grid = Gtk.Grid()
            monitor_grid.set_column_spacing(4)
            monitor_grid.set_row_spacing(4)
            monitor_grid.set_margin_top(2)
            monitor_grid.set_margin_bottom(4)
            monitor_grid.set_margin_start(6)
            monitor_grid.set_margin_end(6)
            monitoring_frame.add(monitor_grid)
            monitor_dial = Dial()
            monitor_dial.set_adjustment(self.monitor_level)
            monitor_grid.attach(label, 0, 0, 3, 1)
            monitor_grid.attach(monitor_dial, 0, 1, 2, 1)

            self.monitor_on = Gtk.Button(label="ON")
            self.monitor_on.get_style_context().add_class('bmdbtn')
            self.monitor_on.connect('clicked', self.do_monitor_on)
            self.monitor_dim = Gtk.Button(label="Dim")
            self.monitor_dim.get_style_context().add_class('bmdbtn')
            self.monitor_dim.connect('clicked', self.do_monitor_dim)
            monitor_grid.attach(self.monitor_on, 0, 2, 1, 1)
            monitor_grid.attach(self.monitor_dim, 1, 2, 1, 1)
            vu_left = Gtk.ProgressBar()
            vu_right = Gtk.ProgressBar()
            vu_left.set_orientation(Gtk.Orientation.VERTICAL)
            vu_left.set_inverted(True)
            vu_right.set_orientation(Gtk.Orientation.VERTICAL)
            vu_right.set_inverted(True)
            vu_box = Gtk.Box()
            vu_box.pack_start(vu_left, False, False, 0)
            vu_box.pack_start(vu_right, False, False, 0)
            self.vu['monitor'] = (vu_left, vu_right)
            monitor_grid.attach(vu_box, 2, 1, 1, 2)

        self.apply_css(self.audio_channels, self.provider)
        self.audio_channels.show_all()

    def on_audio_input_change(self, data):
        self.mixer = 'atem'

        if len(self.volume_level) == 0:
            self.master_level = LogAdjustment(0, 0, 65381, 10, 10, 100)
            self.master_level.connect('value-changed', self.on_master_changed)
            self.monitor_level = LogAdjustment(0, 0, 65381, 10, 10, 100)
            self.monitor_level.connect('value-changed', self.on_monitor_level_changed)
            self.make_mixer_ui(self.connection.mixer.mixerstate['audio-input'])

        self.audio_strip[data.strip_id] = data
        if data.strip_id not in self.volume_level:
            self.volume_level[data.strip_id] = LogAdjustment(0, 65381, 0, 10, 10, 100)
            self.pan[data.strip_id] = Gtk.Adjustment(0, -10000, 10000, 10, 10, 100)
            self.volume_level[data.strip_id].connect('value-changed', self.on_volume_changed)
            self.pan[data.strip_id].source = data.index
            self.pan[data.strip_id].connect('value-changed', self.on_pan_changed)

        self.model_changing = True
        self.volume_level[data.strip_id].set_value_log(data.volume)
        self.pan[data.strip_id].set_value(data.balance)
        if data.strip_id in self.audio_tally:
            tally = self.audio_tally[data.strip_id]
            self.set_class(tally, 'afv', data.state == 2)
        if data.strip_id in self.audio_on:
            self.set_class(self.audio_on[data.strip_id], 'program', data.state == 1)
            self.set_class(self.audio_afv[data.strip_id], 'active', data.state == 2)
        self.model_changing = False

    def on_fairlight_audio_input_change(self, data):
        self.mixer = 'fairlight'
        self.master_level = LogAdjustment(0, -10000, 1100, 10, 10, 100, range=30, exp=True)
        self.master_level.connect('value-changed', self.on_master_changed)
        self.monitor_level = LogAdjustment(0, -10000, 1100, 10, 10, 100, range=30, exp=True)
        self.monitor_level.connect('value-changed', self.on_monitor_level_changed)
        inputs = self.connection.mixer.mixerstate['fairlight-audio-input']
        self.make_mixer_ui(inputs)

    def on_audio_mixer_tally_change(self, data):
        for strip_id in data.tally:
            if strip_id in self.audio_tally:
                self.set_class(self.audio_tally[strip_id], 'program', data.tally[strip_id])

    def on_fairlight_tally_change(self, data):
        for strip_id in data.tally:
            if strip_id in self.audio_tally:
                self.set_class(self.audio_tally[strip_id], 'program', data.tally[strip_id])

    def on_fairlight_strip_properties_change(self, data):
        """
        :type data: FairlightStripPropertiesField
        :return:
        """
        self.audio_strip[data.strip_id] = data
        if data.strip_id not in self.volume_level:
            self.volume_level[data.strip_id] = LogAdjustment(0, -10000, 1000, 10, 10, 100, exp=True)
            self.pan[data.strip_id] = Gtk.Adjustment(0, -10000, 10000, 10, 10, 100)
            self.input_gain[data.strip_id] = LogAdjustment(0, -10000, 600, 10, 10, 100)
            self.delay[data.strip_id] = Gtk.Adjustment(0, 0, 8, 1, 1, 1)
            self.volume_level[data.strip_id].connect('value-changed', self.on_volume_changed)

        self.model_changing = True
        self.volume_level[data.strip_id].set_value_log(data.volume)
        self.pan[data.strip_id].set_value(data.pan)
        self.input_gain[data.strip_id].set_value_log(data.gain)
        self.delay[data.strip_id].set_value(data.delay)
        if data.strip_id in self.audio_tally:
            tally = self.audio_tally[data.strip_id]
            self.set_class(tally, 'afv', data.state == 4)
        if data.strip_id in self.audio_on:
            self.set_class(self.audio_on[data.strip_id], 'program', data.state & 2)
            self.set_class(self.audio_afv[data.strip_id], 'active', data.state & 4)

        self.eq[data.strip_id].set_enabled(data.eq_enable)
        self.model_changing = False

    def on_audio_mixer_master_properties_change(self, data):
        self.model_changing = True
        self.master_level.set_value_log(data.volume)
        self.set_class(self.master_afv, 'active', data.afv)
        self.set_class(self.ftb_afv, 'active', data.afv)
        self.model_changing = False

    def on_fairlight_eq_band_change(self, data):
        if data.strip_id not in self.eq:
            return
        self.eq[data.strip_id].update_band(data)

    def on_volume_changed(self, widget, *args):
        if self.model_changing:
            return
        if self.mixer == 'fairlight':
            cmd = FairlightStripPropertiesCommand(source=widget.source, channel=widget.channel,
                                                  volume=int(widget.get_value_log()))
            self.connection.mixer.send_commands([cmd])
        elif self.mixer == 'atem':
            cmd = AudioInputCommand(source=widget.source, volume=int(widget.get_value_log()))
            self.connection.mixer.send_commands([cmd])

    def on_master_changed(self, widget, *args):
        if self.model_changing:
            return
        if self.mixer == 'fairlight':
            cmd = FairlightMasterPropertiesCommand(volume=int(widget.get_value_log()))
            self.connection.mixer.send_commands([cmd])
        elif self.mixer == 'atem':
            cmd = AudioMasterPropertiesCommand(volume=int(widget.get_value_log()))
            self.connection.mixer.send_commands([cmd])

    def on_monitor_level_changed(self, widget, *args):
        if self.model_changing:
            return
        if self.mixer == 'atem':
            cmd = AudioMonitorPropertiesCommand(volume=int(widget.get_value_log()))
            self.connection.mixer.send_commands([cmd])
        else:
            pass

    def on_pan_changed(self, widget, *args):
        if self.model_changing:
            return
        if self.mixer == 'fairlight':
            cmd = FairlightStripPropertiesCommand(source=widget.source, channel=widget.channel,
                                                  balance=int(widget.get_value()))
            self.connection.mixer.send_commands([cmd])
        elif self.mixer == 'atem':
            cmd = AudioInputCommand(source=widget.source, balance=int(widget.get_value()))
            self.connection.mixer.send_commands([cmd])

    def on_input_gain_changed(self, widget, *args):
        if self.model_changing:
            return
        if self.mixer == 'fairlight':
            cmd = FairlightStripPropertiesCommand(source=widget.source, channel=widget.channel,
                                                  gain=int(widget.get_value_log()))
            self.connection.mixer.send_commands([cmd])

    def do_channel_on(self, widget, *args):
        if self.mixer == 'atem':
            cmd = AudioInputCommand(source=widget.source, on=(not widget.get_style_context().has_class('program')))
        else:
            if widget.get_style_context().has_class('program'):
                state = 1
            else:
                state = 2
            cmd = FairlightStripPropertiesCommand(source=widget.source, channel=widget.channel, state=state)
        self.connection.mixer.send_commands([cmd])

    def do_channel_afv(self, widget, *args):
        if self.mixer == 'atem':
            cmd = AudioInputCommand(source=widget.source, afv=(not widget.get_style_context().has_class('active')))
        else:
            if widget.get_style_context().has_class('active'):
                state = 1
            else:
                state = 4
            cmd = FairlightStripPropertiesCommand(source=widget.source, channel=widget.channel, state=state)
        self.connection.mixer.send_commands([cmd])

    def do_master_afv(self, widget, *args):
        if self.mixer == 'atem':
            cmd = AudioMasterPropertiesCommand(afv=not widget.get_style_context().has_class('active'))
        else:
            cmd = FairlightMasterPropertiesCommand(afv=not widget.get_style_context().has_class('active'))
        self.connection.mixer.send_commands([cmd])

    def do_monitor_on(self, widget, *args):
        if self.mixer == 'atem':
            state = widget.get_style_context().has_class('active')
            cmd = AudioMonitorPropertiesCommand(mute=state)
            self.connection.mixer.send_commands([cmd])
        else:
            pass

    def do_monitor_dim(self, widget, *args):
        if self.mixer == 'atem':
            state = widget.get_style_context().has_class('active')
            cmd = AudioMonitorPropertiesCommand(dim=not state)
            self.connection.mixer.send_commands([cmd])
        else:
            pass

    def on_aux_monitor_source_change(self, data):
        strip_id = f'{data.source}.0'
        if strip_id in self.audio_monitor:
            for mon in self.audio_monitor:
                self.set_class(self.audio_monitor[mon], 'active', mon == strip_id)

    def do_channel_monitor(self, widget, *args):
        # When aux-follow-monitor is enabled send the monitor commands as aux switching commands to
        # the hardware for the selected aux outputs, this will make the aux sdi out have monitor
        # audio for ATEM devices without any real monitor outputs, which is a lot of them
        if len(self.aux_follow_audio) > 0:
            cmd = []
            for bus in self.aux_follow_audio:
                cmd.append(AuxSourceCommand(bus, source=widget.source))
            self.connection.mixer.send_commands(cmd)

        if self.mixer == 'atem':
            state = widget.get_style_context().has_class('active')
            cmd = AudioMonitorPropertiesCommand(solo=not state, solo_source=widget.source)
            self.connection.mixer.send_commands([cmd])
        else:
            pass

    def on_fairlight_master_properties_change(self, data):
        self.model_changing = True
        self.master_level.set_value_log(data.volume)
        self.set_class(self.master_afv, 'active', data.afv)
        self.set_class(self.ftb_afv, 'active', data.afv)
        self.model_changing = False

    def on_audio_monitor_properties_change(self, data):
        self.model_changing = True
        self.monitor_level.set_value_log(data.volume)
        self.set_class(self.monitor_on, 'active', not data.mute)
        self.set_class(self.monitor_dim, 'active', data.dim)
        for m in self.audio_monitor:
            self.audio_monitor[m].set_sensitive(data.enabled)
            en = (m == str(data.solo_source) + ".0") and data.solo
            self.set_class(self.audio_monitor[m], 'active', en)
        self.monitor_on.set_sensitive(data.enabled)
        self.monitor_dim.set_sensitive(data.enabled)
        self.model_changing = False

    def enable_levels(self):
        """ The audio page was opened, request data for the levels """
        if self.mixer == 'atem':
            cmd = SendAudioLevelsCommand(True)
        else:
            cmd = SendFairlightLevelsCommand(True)
        self.connection.mixer.send_commands([cmd])

    def disable_levels(self):
        """ The audio page was closed, stop getting the realtime levels """
        if self.mixer == 'atem':
            cmd = SendAudioLevelsCommand(False)
        else:
            cmd = SendFairlightLevelsCommand(False)
        self.connection.mixer.send_commands([cmd])

    def on_audio_meter_levels_change(self, data):
        if 'master' not in self.vu:
            return
        self.vu['master'][0].set_fraction((data.master[0] + 60) / 60)
        self.vu['master'][1].set_fraction((data.master[1] + 60) / 60)
        self.vu['monitor'][0].set_fraction((data.monitor[0] + 60) / 60)
        self.vu['monitor'][1].set_fraction((data.monitor[1] + 60) / 60)
        for strip in data.input:
            strip_id = f'{strip}.0'
            self.vu[strip_id][0].set_fraction((data.input[strip][0] + 60) / 60)
            self.vu[strip_id][1].set_fraction((data.input[strip][1] + 60) / 60)

    def on_fairlight_meter_levels_change(self, data):
        if data.strip_id not in self.vu:
            return
        self.vu[data.strip_id][0].set_fraction((data.level[0] + 60) / 60)
        self.vu[data.strip_id][1].set_fraction((data.level[1] + 60) / 60)

    def on_fairlight_master_levels_change(self, data):
        if 'master' not in self.vu:
            return
        self.vu['master'][0].set_fraction((data.level[0] + 60) / 60)
        self.vu['master'][1].set_fraction((data.level[1] + 60) / 60)

    def on_open_eq_overlay(self, widget, *args):
        EqWindow(widget.strip_id, self.window, self.connection, self.provider)



#!/usr/bin/env python3
import argparse
import json
import sys

import requests
from requests.auth import HTTPBasicAuth


def switcher(proxy, index, to_source, auto=False, username=None, password=None):
    auth = None
    if username is not None and password is not None:
        auth = HTTPBasicAuth(username, password)

    if to_source.isnumeric():
        to_source = int(to_source)

    if auto:
        r = requests.get(f'{proxy}/program-bus-input', auth=auth)
        try:
            reply = r.json()
            if reply[str(index)]['source'] != to_source:
                data = {'index': index, 'source': to_source}
                requests.post(f'{proxy}/preview-input', data, auth=auth)
                data = {'index': index}
                requests.post(f'{proxy}/auto', data, auth=auth)
            else:
                sys.exit(2)
        except json.decoder.JSONDecodeError:
            sys.exit(1)
    else:
        data = {'index': index, 'source': to_source}
        requests.post(f'{proxy}/program-input', data=data, auth=auth)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--proxy', '-s', required=True)
    parser.add_argument('--to-source', '-t', required=True)
    parser.add_argument('--username', '-u', required=False)
    parser.add_argument('--password', '-p', required=False)
    parser.add_argument('--auto', '-a', required=False, action='store_true')
    parser.add_argument('--index', '-i', type=int, required=False, default=0)
    args = parser.parse_args()

    switcher(args.proxy, args.index, args.to_source, args.auto, args.username, args.password)



# Copyright 2025, Martijn Braam and the OpenAtem contributors
# SPDX-License-Identifier: GPL-3.0-only
from gi.repository import Gtk, GObject, Gdk

from gtk_switcher.palettepanel import PalettePanel
from pyatem.command import MediaplayerSelectCommand


class MediaPlayerPanel(PalettePanel):
    __gtype_name__ = 'MediaPlayerpanel'

    def __init__(self, index, model_media, connection):
        self.index = index
        self.model_media = model_media
        self.connection = connection

        self.name = _("Media Player {}").format(index + 1)
        super().__init__(self.name, connection)

        self.dropdown = self.add_control_dropdown(_("Media"), self.model_media, handler=self.on_mediaplayer_change)

        self.show_all()

    def __repr__(self):
        return '<MediaPlayerPanel {}>'.format(self.panel_name)

    @PalettePanel.event('change:mediaplayer-selected:*')
    def on_mediaplayer_switcher_source_change(self, data):
        if data.index != self.index:
            return

        with self.model:
            if data.source_type == 1:
                self.dropdown.set_active_id(str(data.slot))

    def on_mediaplayer_change(self, widget):
        index = widget.get_active_id()
        if index == "":
            return

        index = int(index)
        self.run(MediaplayerSelectCommand(self.index, still=index))



# Copyright 2021 - 2022, Martijn Braam and the OpenAtem contributors
# SPDX-License-Identifier: LGPL-3.0-only
import socket


class VideoHub:
    _SECTION_UNKNOWN = 0
    _SECTION_PREAMBLE = 1
    _SECTION_DEVICE = 2
    _SECTION_INPUTLABEL = 3
    _SECTION_INPUTSTATUS = 4
    _SECTION_OUTPUTLABEL = 5
    _SECTION_CONFIGURATION = 6
    _SECTION_ROUTING = 7
    _SECTION_END = 8

    def __init__(self, ip, port=9990):
        self.ip = ip
        self.port = port
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.version = None

        self._section = VideoHub._SECTION_PREAMBLE
        self._sections = {
            'PROTOCOL PREAMBLE:': VideoHub._SECTION_PREAMBLE,
            'VIDEOHUB DEVICE:': VideoHub._SECTION_DEVICE,
            'INPUT LABELS:': VideoHub._SECTION_INPUTLABEL,
            'VIDEO INPUT STATUS:': VideoHub._SECTION_INPUTSTATUS,
            'OUTPUT LABELS:': VideoHub._SECTION_OUTPUTLABEL,
            'CONFIGURATION:': VideoHub._SECTION_CONFIGURATION,
            'VIDEO OUTPUT ROUTING:': VideoHub._SECTION_ROUTING,
            'END PRELUDE:': VideoHub._SECTION_END,
        }

        self.model = 'Unknown'
        self.model_display = 'Unknown'
        self.uuid = ''
        self.mode = None
        self.input_count = 0
        self.output_count = 0
        self.input_label = {}
        self.input_status = {}
        self.output_label = {}
        self.output_source = {}

        self._handler = {
            'connect': set(),
            'disconnect': set(),
            'route-change': set(),
            'input-label-change': set(),
            'output-label-change': set(),
            'input-status-change': set(),
        }

    def connect(self):
        self.sock.connect((self.ip, self.port))
        self.sock.settimeout(15)
        line = self._readline()
        if line != 'PROTOCOL PREAMBLE:':
            raise ValueError("Not a videohub")
        version = self._readline()
        _, self.version = version.split(': ', maxsplit=1)

    def _readline(self):
        buffer = b''
        while True:
            char = self.sock.recv(1)
            if char == b'\n':
                return buffer.decode()
            buffer += char

    def on(self, event, handler):
        if event not in self._handler:
            raise ValueError("Unknown event")
        self._handler[event].add(handler)

    def _raise(self, event, **kwargs):
        for handler in self._handler[event]:
            handler(self, **kwargs)

    def _parse_device(self, line):
        key, value = line.split(': ', maxsplit=1)
        if key == 'Model name':
            self.model = value
        elif key == 'Friendly name':
            self.model_display = value
        elif key == 'Unique ID':
            self.uuid = value
        elif key == 'Video inputs':
            self.input_count = int(value)
        elif key == 'Video outputs':
            self.output_count = int(value)

    def _parse_configuration(self, line):
        key, value = line.split(': ', maxsplit=1)
        if key == 'Video Mode':
            self.mode = value

    def _parse_inputlabel(self, line):
        key, value = line.split(' ', maxsplit=1)
        index = int(key)
        self.input_label[index] = value
        self._raise('input-label-change', index=index, label=value)

    def _parse_inputstatus(self, line):
        key, value = line.split(' ', maxsplit=1)
        index = int(key)
        self.input_status[index] = value
        self._raise('input-status-change', index=index, status=value)

    def _parse_outputlabel(self, line):
        key, value = line.split(' ', maxsplit=1)
        index = int(key)
        self.output_label[index] = value
        self._raise('output-label-change', index=index, label=value)

    def _parse_routing(self, line):
        if line == 'ACK':
            return
        key, value = line.split(' ', maxsplit=1)
        index = int(key)
        source = int(value)
        self.output_source[index] = source
        self._raise('route-change', index=index, source=source)

    def _ping(self):
        self.sock.send(b'PING:\n\n')

    def loop(self):
        try:
            line = self._readline()
        except TimeoutError:
            # No traffic for 15 seconds, try to send a ping
            self._ping()
            return

        if line == '' or line == 'ACK':
            return

        if line in self._sections:
            self._section = self._sections[line]
            if self._section == VideoHub._SECTION_END:
                self._raise('connect')
            return
        elif line.endswith(':'):
            self._section = VideoHub._SECTION_UNKNOWN
            return

        if self._section == VideoHub._SECTION_DEVICE:
            return self._parse_device(line)
        elif self._section == VideoHub._SECTION_INPUTLABEL:
            return self._parse_inputlabel(line)
        elif self._section == VideoHub._SECTION_INPUTSTATUS:
            return self._parse_inputstatus(line)
        elif self._section == VideoHub._SECTION_OUTPUTLABEL:
            return self._parse_outputlabel(line)
        elif self._section == VideoHub._SECTION_CONFIGURATION:
            return self._parse_configuration(line)
        elif self._section == VideoHub._SECTION_ROUTING:
            return self._parse_routing(line)

    def set_source(self, index, source):
        cmd = f'VIDEO OUTPUT ROUTING:\n{index} {source}\n\n'
        self.sock.send(cmd.encode())


if __name__ == '__main__':
    test = VideoHub('192.168.2.84')
    test.connect()


    def on_connect(device):
        print(device.model)
        device.set_source(1, 4)


    def on_route(device, index, source):
        print(f"Route: {device.input_label[source]} -> {device.output_label[index]}")


    test.on('connect', on_connect)
    test.on('route-change', on_route)
    while True:
        test.loop()



#!/usr/bin/env python
# -*- coding: latin-1 -*-

# <-- removing this magic comment breaks Python 3.4 on Windows
"""
1. Dump binary data to the following text format:

00000000: 00 00 00 5B 68 65 78 64  75 6D 70 5D 00 00 00 00  ...[hexdump]....
00000010: 00 11 22 33 44 55 66 77  88 99 AA BB CC DD EE FF  .."3DUfw........

It is similar to the one used by:
Scapy
00 00 00 5B 68 65 78 64 75 6D 70 5D 00 00 00 00  ...[hexdump]....
00 11 22 33 44 55 66 77 88 99 AA BB CC DD EE FF  .."3DUfw........

Far Manager
000000000: 00 00 00 5B 68 65 78 64 ¦ 75 6D 70 5D 00 00 00 00     [hexdump]
000000010: 00 11 22 33 44 55 66 77 ¦ 88 99 AA BB CC DD EE FF   ?"3DUfwˆ™ª»ÌÝîÿ


2. Restore binary data from the formats above as well
   as from less exotic strings of raw hex

"""

__version__ = '3.3'
__author__ = 'anatoly techtonik <techtonik@gmail.com>'
__license__ = 'Public Domain'

__history__ = \
    """
    3.3 (2015-01-22)
     * accept input from sys.stdin if "-" is specified
       for both dump and restore (issue #1)
     * new normalize_py() helper to set sys.stdout to
       binary mode on Windows
    
    3.2 (2015-07-02)
     * hexdump is now packaged as .zip on all platforms
       (on Linux created archive was tar.gz)
     * .zip is executable! try `python hexdump-3.2.zip`
     * dump() now accepts configurable separator, patch
       by Ian Land (PR #3)
    
    3.1 (2014-10-20)
     * implemented workaround against mysterious coding
       issue with Python 3 (see revision 51302cf)
     * fix Python 3 installs for systems where UTF-8 is
       not default (Windows), thanks to George Schizas
       (the problem was caused by reading of README.txt)
    
    3.0 (2014-09-07)
     * remove unused int2byte() helper
     * add dehex(text) helper to convert hex string
       to binary data
     * add 'size' argument to dump() helper to specify
       length of chunks
    
    2.0 (2014-02-02)
     * add --restore option to command line mode to get
       binary data back from hex dump
     * support saving test output with `--test logfile`
     * restore() from hex strings without spaces
     * restore() now raises TypeError if input data is
       not string
     * hexdump() and dumpgen() now don't return unicode
       strings in Python 2.x when generator is requested
    
    1.0 (2013-12-30)
     * length of address is reduced from 10 to 8
     * hexdump() got new 'result' keyword argument, it
       can be either 'print', 'generator' or 'return'
     * actual dumping logic is now in new dumpgen()
       generator function
     * new dump(binary) function that takes binary data
       and returns string like "66 6F 72 6D 61 74"
     * new genchunks(mixed, size) function that chunks
       both sequences and file like objects
    
    0.5 (2013-06-10)
     * hexdump is now also a command line utility (no
       restore yet)
    
    0.4 (2013-06-09)
     * fix installation with Python 3 for non English
       versions of Windows, thanks to George Schizas
    
    0.3 (2013-04-29)
     * fully Python 3 compatible
    
    0.2 (2013-04-28)
     * restore() to recover binary data from a hex dump in
       native, Far Manager and Scapy text formats (others
       might work as well)
     * restore() is Python 3 compatible
    
    0.1 (2013-04-28)
     * working hexdump() function for Python 2
    """

import binascii  # binascii is required for Python 3
import sys

# --- constants
PY3K = sys.version_info >= (3, 0)


# --- workaround against Python consistency issues
def normalize_py():
    ''' Problem 001 - sys.stdout in Python is by default opened in
        text mode, and writes to this stdout produce corrupted binary
        data on Windows

            python -c "import sys; sys.stdout.write('_\n_')" > file
            python -c "print(repr(open('file', 'rb').read()))"
    '''
    if sys.platform == "win32":
        # set sys.stdout to binary mode on Windows
        import os, msvcrt
        msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)


# --- - chunking helpers
def chunks(seq, size):
    '''Generator that cuts sequence (bytes, memoryview, etc.)
       into chunks of given size. If `seq` length is not multiply
       of `size`, the lengh of the last chunk returned will be
       less than requested.

       >>> list( chunks([1,2,3,4,5,6,7], 3) )
       [[1, 2, 3], [4, 5, 6], [7]]
    '''
    d, m = divmod(len(seq), size)
    for i in range(d):
        yield seq[i * size:(i + 1) * size]
    if m:
        yield seq[d * size:]


def chunkread(f, size):
    '''Generator that reads from file like object. May return less
       data than requested on the last read.'''
    c = f.read(size)
    while len(c):
        yield c
        c = f.read(size)


def genchunks(mixed, size):
    '''Generator to chunk binary sequences or file like objects.
       The size of the last chunk returned may be less than
       requested.'''
    if hasattr(mixed, 'read'):
        return chunkread(mixed, size)
    else:
        return chunks(mixed, size)


# --- - /chunking helpers


def dehex(hextext):
    """
    Convert from hex string to binary data stripping
    whitespaces from `hextext` if necessary.
    """
    if PY3K:
        return bytes.fromhex(hextext)
    else:
        hextext = "".join(hextext.split())
        return hextext.decode('hex')


def dump(binary, size=2, sep=' '):
    '''
    Convert binary data (bytes in Python 3 and str in
    Python 2) to hex string like '00 DE AD BE EF'.
    `size` argument specifies length of text chunks
    and `sep` sets chunk separator.
    '''
    hexstr = binascii.hexlify(binary)
    if PY3K:
        hexstr = hexstr.decode('ascii')
    return sep.join(chunks(hexstr.upper(), size))


def dumpgen(data):
    '''
    Generator that produces strings:

    '00000000: 00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00  ................'
    '''
    generator = genchunks(data, 16)
    for addr, d in enumerate(generator):
        # 00000000:
        line = '%08X: ' % (addr * 16)
        # 00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
        dumpstr = dump(d)
        line += dumpstr[:8 * 3]
        if len(d) > 8:  # insert separator if needed
            line += ' ' + dumpstr[8 * 3:]
        # ................
        # calculate indentation, which may be different for the last line
        pad = 2
        if len(d) < 16:
            pad += 3 * (16 - len(d))
        if len(d) <= 8:
            pad += 1
        line += ' ' * pad

        for byte in d:
            # printable ASCII range 0x20 to 0x7E
            if not PY3K:
                byte = ord(byte)
            if 0x20 <= byte <= 0x7E:
                line += chr(byte)
            else:
                line += '.'
        yield line


def hexdump(data, result='print'):
    '''
    Transform binary data to the hex dump text format:

    00000000: 00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00  ................

      [x] data argument as a binary string
      [x] data argument as a file like object

    Returns result depending on the `result` argument:
      'print'     - prints line by line
      'return'    - returns single string
      'generator' - returns generator that produces lines
    '''
    if PY3K and type(data) == str:
        raise TypeError('Abstract unicode data (expected bytes sequence)')

    gen = dumpgen(data)
    if result == 'generator':
        return gen
    elif result == 'return':
        return '\n'.join(gen)
    elif result == 'print':
        for line in gen:
            print(line)
    else:
        raise ValueError('Unknown value of `result` argument')


def restore(dump):
    '''
    Restore binary data from a hex dump.
      [x] dump argument as a string
      [ ] dump argument as a line iterator

    Supported formats:
      [x] hexdump.hexdump
      [x] Scapy
      [x] Far Manager
    '''
    minhexwidth = 2 * 16  # minimal width of the hex part - 00000... style
    bytehexwidth = 3 * 16 - 1  # min width for a bytewise dump - 00 00 ... style

    result = bytes() if PY3K else ''
    if type(dump) != str:
        raise TypeError('Invalid data for restore')

    text = dump.strip()  # ignore surrounding empty lines
    for line in text.split('\n'):
        # strip address part
        addrend = line.find(':')
        if 0 < addrend < minhexwidth:  # : is not in ascii part
            line = line[addrend + 1:]
        line = line.lstrip()
        # check dump type
        if line[2] == ' ':  # 00 00 00 ...  type of dump
            # check separator
            sepstart = (2 + 1) * 7 + 2  # ('00'+' ')*7+'00'
            sep = line[sepstart:sepstart + 3]
            if sep[:2] == '  ' and sep[2:] != ' ':  # ...00 00  00 00...
                hexdata = line[:bytehexwidth + 1]
            elif sep[2:] == ' ':  # ...00 00 | 00 00...  - Far Manager
                hexdata = line[:sepstart] + line[sepstart + 3:bytehexwidth + 2]
            else:  # ...00 00 00 00... - Scapy, no separator
                hexdata = line[:bytehexwidth]
            line = hexdata
        result += dehex(line)
    return result


def runtest(logfile=None):
    '''Run hexdump tests. Requires hexfile.bin to be in the same
       directory as hexdump.py itself'''

    class TeeOutput(object):
        def __init__(self, stream1, stream2):
            self.outputs = [stream1, stream2]

        # -- methods from sys.stdout / sys.stderr
        def write(self, data):
            for stream in self.outputs:
                if PY3K:
                    if 'b' in stream.mode:
                        data = data.encode('utf-8')
                stream.write(data)
                stream.flush()

        def tell(self):
            raise IOError

        def flush(self):
            for stream in self.outputs:
                stream.flush()
        # --/ sys.stdout

    if logfile:
        openlog = open(logfile, 'wb')
        # copy stdout and stderr streams to log file
        savedstd = sys.stderr, sys.stdout
        sys.stderr = TeeOutput(sys.stderr, openlog)
        sys.stdout = TeeOutput(sys.stdout, openlog)

    def echo(msg, linefeed=True):
        sys.stdout.write(msg)
        if linefeed:
            sys.stdout.write('\n')

    expected = '''\
00000000: 00 00 00 5B 68 65 78 64  75 6D 70 5D 00 00 00 00  ...[hexdump]....
00000010: 00 11 22 33 44 55 66 77  88 99 0A BB CC DD EE FF  .."3DUfw........\
'''

    # get path to hexfile.bin
    # this doesn't work from .zip
    #   import os.path as osp
    #   hexfile = osp.dirname(osp.abspath(__file__)) + '/hexfile.bin'
    # this doesn't work either
    #   hexfile = osp.dirname(sys.modules[__name__].__file__) + '/hexfile.bin'
    # this works
    import pkgutil
    bin = pkgutil.get_data('hexdump', 'data/hexfile.bin')

    # varios length of input data
    hexdump(b'zzzz' * 12)
    hexdump(b'o' * 17)
    hexdump(b'p' * 24)
    hexdump(b'q' * 26)
    # allowable character set filter
    hexdump(b'line\nfeed\r\ntest')
    hexdump(b'\x00\x00\x00\x5B\x68\x65\x78\x64\x75\x6D\x70\x5D\x00\x00\x00\x00'
            b'\x00\x11\x22\x33\x44\x55\x66\x77\x88\x99\x0A\xBB\xCC\xDD\xEE\xFF')
    print('---')
    # dumping file-like binary object to screen (default behavior)
    hexdump(bin)
    print('return output')
    hexout = hexdump(bin, result='return')
    assert hexout == expected, 'returned hex didn\'t match'
    print('return generator')
    hexgen = hexdump(bin, result='generator')
    assert next(hexgen) == expected.split('\n')[0], 'hex generator 1 didn\'t match'
    assert next(hexgen) == expected.split('\n')[1], 'hex generator 2 didn\'t match'

    # binary restore test
    bindata = restore(
        '''
        00000000: 00 00 00 5B 68 65 78 64  75 6D 70 5D 00 00 00 00  ...[hexdump]....
        00000010: 00 11 22 33 44 55 66 77  88 99 0A BB CC DD EE FF  .."3DUfw........
        ''')
    echo('restore check ', linefeed=False)
    assert bin == bindata, 'restore check failed'
    echo('passed')

    far = \
        '''
        000000000: 00 00 00 5B 68 65 78 64 ¦ 75 6D 70 5D 00 00 00 00     [hexdump]
        000000010: 00 11 22 33 44 55 66 77 ¦ 88 99 0A BB CC DD EE FF   ?"3DUfwˆ™ª»ÌÝîÿ
        '''
    echo('restore far format ', linefeed=False)
    assert bin == restore(far), 'far format check failed'
    echo('passed')

    scapy = '''\
00 00 00 5B 68 65 78 64 75 6D 70 5D 00 00 00 00  ...[hexdump]....
00 11 22 33 44 55 66 77 88 99 0A BB CC DD EE FF  .."3DUfw........
'''
    echo('restore scapy format ', linefeed=False)
    assert bin == restore(scapy), 'scapy format check failed'
    echo('passed')

    if not PY3K:
        assert restore('5B68657864756D705D') == '[hexdump]', 'no space check failed'
        assert dump('\\\xa1\xab\x1e', sep='').lower() == '5ca1ab1e'
    else:
        assert restore('5B68657864756D705D') == b'[hexdump]', 'no space check failed'
        assert dump(b'\\\xa1\xab\x1e', sep='').lower() == '5ca1ab1e'

    print('---[test file hexdumping]---')

    import os
    import tempfile
    hexfile = tempfile.NamedTemporaryFile(delete=False)
    try:
        hexfile.write(bin)
        hexfile.close()
        hexdump(open(hexfile.name, 'rb'))
    finally:
        os.remove(hexfile.name)
    if logfile:
        sys.stderr, sys.stdout = savedstd
        openlog.close()


def main():
    from optparse import OptionParser
    parser = OptionParser(usage='''
  %prog [binfile|-]
  %prog -r hexfile
  %prog --test [logfile]''', version=__version__)
    parser.add_option('-r', '--restore', action='store_true',
                      help='restore binary from hex dump')
    parser.add_option('--test', action='store_true', help='run hexdump sanity checks')

    options, args = parser.parse_args()

    if options.test:
        if args:
            runtest(logfile=args[0])
        else:
            runtest()
    elif not args or len(args) > 1:
        parser.print_help()
        sys.exit(-1)
    else:
        ## dump file
        if not options.restore:
            # [x] memory effective dump
            if args[0] == '-':
                if not PY3K:
                    hexdump(sys.stdin)
                else:
                    hexdump(sys.stdin.buffer)
            else:
                hexdump(open(args[0], 'rb'))

        ## restore file
        else:
            # prepare input stream
            if args[0] == '-':
                instream = sys.stdin
            else:
                if PY3K:
                    instream = open(args[0])
                else:
                    instream = open(args[0], 'rb')

            # output stream
            # [ ] memory efficient restore
            if PY3K:
                sys.stdout.buffer.write(restore(instream.read()))
            else:
                # Windows - binary mode for sys.stdout to prevent data corruption
                normalize_py()
                sys.stdout.write(restore(instream.read()))


if __name__ == '__main__':
    main()

# [x] file restore from command line utility
# [ ] write dump with LF on Windows for consistency
# [ ] encoding param for hexdump()ing Python 3 str if anybody requests that

# [ ] document chunking API
# [ ] document hexdump API
# [ ] blog about sys.stdout text mode problem on Windows



# Copyright 2025, Martijn Braam and the OpenAtem contributors
# SPDX-License-Identifier: GPL-3.0-only
from gi.repository import Gtk, GObject, Gdk

from gtk_switcher.palettepanel import PalettePanel
from pyatem.command import ColorGeneratorCommand


class ColorPanel(PalettePanel):
    __gtype_name__ = 'ColorPanel'

    def __init__(self, connection):
        self.connection = connection
        self.name = _("Color Generators")

        super().__init__(self.name, connection, preset_domain="colors", preset_fields=['color-generator'])

        self.colors = [
            self.add_control_color(_("Color {}").format(1), handler=self.on_color_set, handler_args=[0]),
            self.add_control_color(_("Color {}").format(2), handler=self.on_color_set, handler_args=[1]),
        ]
        self.show_all()

    def __repr__(self):
        return '<ColorPanel {}>'.format(self.panel_name)

    @PalettePanel.event('change:color-generator:*')
    def on_color_change(self, data):
        r, g, b = data.get_rgb()
        color = Gdk.RGBA()
        color.red = r
        color.green = g
        color.blue = b
        color.alpha = 1.0
        self.colors[data.index].set_rgba(color)

    def on_color_set(self, widget, index):
        color = widget.get_rgba()
        cmd = ColorGeneratorCommand.from_rgb(index=index, red=color.red, green=color.green, blue=color.blue)
        self.run(cmd)
